#include "AgentManagerRes.h"
#include "AgentManager.h"

CAgentManager::CAgentManager()
{
	m_iAgentStatus = GetAgentStatus();
}

CAgentManager::~CAgentManager()
{
}

//********************************//
//  Return value:
//      None.
//  Description:
//      Print the warning string.
//********************************//
void CAgentManager::PrintWarning(char *pszWarning)
{
	printf("\t%s", pszWarning);
	while(getchar() != '\n');
	printf("\n");
}

//****************************************************************************************************//
//  Description:
//  Support the user to select a communication port in auto searching way or in manual selecting way.
//****************************************************************************************************//
void CAgentManager::DoSelectMode()
{
	int nInputValue;
	int nFlag = 0;
	
	while(1)
    {
        printf("\n");
        printf("\t%s\n\t%s\n\t%s\n", pStrUI[7],pStrUI[15], pStrUI[8]);
        printf("\t0. %s\n", pStrUI[3]);
        printf("\n\t%s", pStrUI[4]);

        nInputValue = CheckInput(3);

        switch (nInputValue)
        {
            case -1:
                break;
            case 0:
                nFlag = 1;
                break;
            case 1: //AutoMode
                nFlag = 1;
                m_cfg.GetShutdownParameter(&ShutdownParam);
                ShutdownParam.AutoMode = AUTO_SEL;
                ShutdownParam.uComPort = 0;
                m_cfg.SetShutdownParameter(ShutdownParam);
                ManSelect();
                break;
            case 2: //AutoMode
                nFlag = 1;
                m_cfg.GetShutdownParameter(&ShutdownParam);
                ShutdownParam.AutoMode = AUTOMODE_ALL_USB_WITHOUT_COM;
                ShutdownParam.uComPort = 0;
                m_cfg.SetShutdownParameter(ShutdownParam);
                ManSelect();
                break;
            case 3: //Manual Mode
                nFlag = 1;
                SetComPort();
                break;
            default:
                PrintWarning(pStrUI[5]);
                break;
        }

        if (nFlag == 1)
            break;
    }
}

//***********************************************************************//
//  Description:
//      Implement the function: user select communication port manually.
//***********************************************************************//
void CAgentManager::SetComPort()
{
    int ComList[256];
    int nComCount = 0;
    int nInputValue;

    memset(ComList, 0, sizeof(ComList));
    nComCount = m_ups.GetComPortCount(ComList);

    while (1)
    {
        printf("\n\t%s\n",pStrUI[10]);
        for (int i = 0; i < nComCount; i++)
        {
            printf("\t%d. COM%d\n", (i+1), ComList[i]+1);
        }
        printf("\t0. %s\n", pStrUI[3]);
        printf("\n\t%s", pStrUI[4]);

        nInputValue = CheckInput(nComCount);
        if (nInputValue < 0)
        {
            printf("\t%s",pStrUI[9]);
            continue;
        }
        else if (nInputValue == 0)
            break;

        m_cfg.GetShutdownParameter(&ShutdownParam);
        ShutdownParam.AutoMode = MANUAL_SEL;
        ShutdownParam.uPortType = 1;
        ShutdownParam.uComPort = nInputValue;
        m_cfg.SetShutdownParameter(ShutdownParam);
        ManSelect();

        break;
    }
}

//****************************************************************************************************//
//  Parameters:
//      nMaxNum
//          [in] The max num user can set.
//  Return value:
//      If -1, failed; otherwise return user input number.
//  Description:
//      Get the user input num, and if the num is not within 0-nMaxNum, Warning string will poped up.
//****************************************************************************************************//
int CAgentManager::CheckInput(int nMaxNum)
{
	char szBuf[INPUT_NUM_MAX];
	int iRet; 
	int iValue;
	
	memset(szBuf, 0, sizeof(szBuf));
	iRet = ReadStdInput(szBuf, INPUT_NUM_MAX-1);
	
	if (iRet < 0)
	{
		PrintWarning(pStrUI[5]);
		return -1;
	}
	if (iRet == 1)
		return -1;
	
	iValue = atoi(szBuf);
	
	if (iValue > nMaxNum)
	{
		PrintWarning(pStrUI[5]);
		return -1;
	}
	
	return iValue;
}

//**************************************************************************************************************//
//  Description:
//      Print the communication port information, and support the user to select a communication port manually.
//**************************************************************************************************************//
void CAgentManager::PrintCommPort()
{
	CConfigFile 		cfg;
	SHUTDOWNPARAM		SDParam;
	char 	szBuf [64] = {0};
	
	memset(&SDParam, 0, sizeof(SHUTDOWNPARAM));
	cfg.GetShutdownParameter(&SDParam);
	
	if (SDParam.uPortType == USB)
		sprintf(szBuf, "\tUPS connect to USB\n");
	else
		sprintf(szBuf, "\tUPS connect to COM%d\n", SDParam.uComPort);
	
	printf(szBuf);
	DoSelectMode();
}

//*******************************************************//
//  Parameters:
//      pszBuf
//          [in] The data user input.
//      iLength
//          [in] T he max length user can input;
//  Return value:
//      0: OK, 1: Enter key, -1: user input not number.
//  Description:
//      Get user input number.
//*******************************************************//
int CAgentManager::ReadStdInput(char *pszBuf, int iLength)
{
	int 	iRet = 0;
	int  	i = 0;
	int  	iCount = 0;
	char 	ch = 0;

	if ((iLength == 0) || (pszBuf == NULL))
		return -1;

	fflush(stdout);

	while ((iRet = read(STDIN_FILENO, &ch, 1)) == 1)
	{
		ch &= 0xFF;
		if (ch == '\n')
			break;

		if (iCount <= (iLength-2))
		{
			fflush(stdout);
			pszBuf[iCount++] = ch;
		}
	}

	if ((ch == '\n') && (iCount == 0)) //unsafe
		return 1;

	if ((iCount > 1) && (pszBuf[0] == '0'))
		return -1;

	for (i = 0; i < iCount; i++)
	{
		if ((pszBuf[i] < '0') || (pszBuf[i] > '9'))
			return -1;
	}

	if (iRet <= 0)
		return -1;

	return 0;
}

//******************************//
//  Description:
//      Print the Manager menu.
//******************************//
void CAgentManager::PrintAgentManager()
{
	m_iAgentStatus = GetAgentStatus();
	if (m_iAgentStatus == 1)
		printf("\n\t1. %s\n", pStrUI[0]);
	else
		printf("\n\t1. %s\n", pStrUI[1]);

	printf("\t2. %s\n", pStrUI[2]);
	printf("\t3. %s\n", pStrUI[11]);//martin add 2009-04-01 
	printf("\t0. %s\n", pStrUI[3]);
	printf("\n\t%s", pStrUI[4]);
}

//**********************************//
//  Description:
//      Implement the Manager menu.
//**********************************//
void CAgentManager::UpdateShow()
{
	char szInputBuf[INPUT_NUM_MAX] = {0};
	int nInputValue = 0;
	int	iRet = 0;
	int	iCount = 0;
    int nLoop = 1; //modified by martin begin 2009-04-01
  
	while (1)
	{
		PrintAgentManager();
		memset(szInputBuf, 0, sizeof(szInputBuf));
		iRet = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1); 
		if (iRet < 0)
		{
			PrintWarning(pStrUI[5]);
			continue;
		}
		if (iRet == 1)
			continue;
	
		nInputValue = atoi(szInputBuf);

		/*char mc[200];
        sprintf(mc, " ##leon - CAgentManager::UpdateShow() - nInputValue=%d", nInputValue);
        LogOneMsg(mc);*/

		switch (nInputValue) //modified by martin begin 2009-04-01 
		{
			case 0:
				nLoop = 0;
			    break;
			case 1:
				if (m_iAgentStatus == 1)
					StartAgent();
				else
				{
					StopAgent();
					iCount = 0;
					while (iCount++ < 5)
					{
						m_public.DelayInUsec(1, 0);
						if (GetAgentStatus() == 1)
							break;
					}
				}
			  break;
			case 2:
				if (m_iAgentStatus == 1)
					PrintWarning(pStrUI[6]);
				else
					PrintCommPort();
			    break;
			case 3:
				printf("\n");
			    printf("\t%s\n", pStrUI[12]);
			    printf("\t%s\n", pStrUI[13]);
			    printf("\t%s\n", pStrUI[14]);
			    break;
            default:
                PrintWarning(pStrUI[5]);
                break;
		} 
		if (nLoop == 0)
		  break; //modified by martin end 2009-04-01 
	
		m_public.DelayInUsec(1, 0);
	}
}

//***********************************************************************//
//  Return value:
//      Return 0, the service is running; 1, the service is not running.
//  Description:
//      Check if the daemon service "ssdDaemon" started up.
//***********************************************************************//
int CAgentManager::GetAgentStatus()
{
	int iStatus = 0;
	int	iActive = 0;
	char szCmd[256] = {0};
	
	sprintf(szCmd, "ps -C %s >/dev/null", BIN_FILE_SERVICE);
	iStatus = system(szCmd);
	iActive = m_public.IsServiceActive(LOCKFILE_SERVICE);

	/*char mc[200];
    sprintf(mc, " ##leon - CAgentManager::GetAgentStatus() - iStatus=%d, iActive=%d", iStatus, iActive);
    LogOneMsg(mc);*/

	if ((iStatus <= 0) && (iActive == 1))
		return 0;
	
	return 1;
}

//**************************************//
//  Description:
//      Started up daemon "ssdDaemon".
//**************************************//
void CAgentManager::StartAgent()
{
	int i = 0;
	char szCmd[256] = {0};
	
	for (i = 0; i < 32; i++)
	{
		signal(i, SIG_IGN);
	}
	
    if (fork() == 0)
	{
		sprintf(szCmd, "%s/%s", AGENT_PATH, BIN_FILE_SERVICE);	
		m_public.System(szCmd);
		_exit(0);
	}
}

//*******************************//
//  Description:
//      Stop daemon "ssdDaemon".
//*******************************//
void CAgentManager::StopAgent()
{
	SendSignal(BIN_FILE_SERVICE);
}

void CAgentManager::ManSelect()
{
	int i = 0;
	
	for (i = 0; i < 32; i++)
	{
		signal(i, SIG_IGN);
	}
	
  if(fork() == 0)
	{
		for(i=0; i<128; i++)
		{
			close(i);
		}
		
		execlp("killall", "killall", "-USR2", BIN_FILE_AGENT, NULL);
		_exit(0);
	}
}

void CAgentManager::SendSignal(const char* szProcDes)
{
	int i = 0;
	
	for (i = 0; i < 32; i++)
	{
		signal(i, SIG_IGN);
	}
	
    if (fork() == 0)
	{
		for (i = 0; i < 128; i++)
		{
			close(i);
		}
		
		execlp("killall", "killall", "-USR1", szProcDes, NULL);
		_exit(0);
	}
}
	
int main(int argc, char **argv)
{
	CPublic 		pub;
	CAgentManager 	AgentManager;

	if ((argc == 2) && !strcmp(argv[1], "stop"))
	{
		AgentManager.StopAgent();
		return 1;
	}
	else if (argc >= 2)
		printf("         paramter error!\n");

	if (pub.IsServiceActive(LOCKFILE_AGENTMANAGER))
		return -1;
	
	AgentManager.UpdateShow();
	return 0;	
}
