#include "../Public/ConfigFile.h"
#include "../Public/ups.h"
#include "UserConfigRes.h"
#include "UserConfig.h"
#include <string.h>

#define LOCKFILE_CONFIG		AGENT_PATH"/UserConfig.Lock"

CUserConfig::CUserConfig()
{
	memset(m_szHostName, 0, sizeof(m_szHostName));
	memset(m_iOSPortNo, 0, sizeof(m_iOSPortNo));
	InitShutdownParam();
	gethostname(m_szHostName, 256);
}

CUserConfig::~CUserConfig()
{
}

//****************************************************//
//  Description:
//      Set the shutdown parameters to default value.
//****************************************************//
void CUserConfig::InitShutdownParam()
{
	CConfigFile cfg;
	SHUTDOWNPARAM	oldSDParam;
	cfg.GetShutdownParameter(&oldSDParam);
	memset(&m_ShutdownParam, 0, sizeof(m_ShutdownParam));
	cfg.SetDefaultShutdownParam(&m_ShutdownParam);
	m_ShutdownParam.uPortType = oldSDParam.uPortType;
	m_ShutdownParam.uComPort = oldSDParam.uComPort;
}

int CUserConfig::ClearScreen()
{
	return system("clear");
}

void CUserConfig::PrintNewLine()
{
	printf("\n");
}

void CUserConfig::PrintBlank(int iNum)
{
	int i;
	for (i = 0; i < iNum; i++)
	{
		printf(" ");
	}
}

void CUserConfig::PrintString(char *pStr)
{
	if (pStr != NULL)
		printf("%s", pStr);
}

void CUserConfig::PrintWarning()
{
	PrintBlank(6);
	PrintString(pStrUI[43]);
	while(getchar() != '\n');
	PrintNewLine();  
}

//********************************************//
//  Description:
//      Print configuring information string.
//********************************************//
void CUserConfig::PrintConfigTitle()
{
	char szBuf[512] = {0};

	PrintString(pStrUI[0]);
	PrintString(pStrUI[1]);
	PrintString(pStrUI[2]);
	PrintString(pStrUI[1]);

	sprintf(szBuf, pStrUI[3], m_szHostName);
	PrintString(szBuf);
	PrintString(pStrUI[1]);
}

int CUserConfig::ReadStdInput(char *pszBuf, int iLength)
{
	int iRet = 0;
	int i = 0;
	int iCount = 0;
	char ch = 0;
	char tmp_buf[201] = {0};

	if ((iLength == 0) || (pszBuf == NULL))
		return -1;

	fflush(stdout);

	while ((iRet = read(STDIN_FILENO, &ch, 1)) == 1)
	{
		ch &= 0xFF;
		if (ch == '\n' || iCount > iLength-2)
			break;

		if (iCount <= (iLength-2))
		{
			fflush(stdout);
			pszBuf[iCount] = ch;
		}

		iCount++;
	}

	pszBuf[iCount] = '\0';

	if ((ch == '\n') && (iCount == 0))
		return 1;

	if ((iCount > 1) && (pszBuf[0] == '0'))
		return -1;

	for (i = 0; i < iCount; i++)
	{
		if ((pszBuf[i] < '0') || (pszBuf[i] > '9'))
			return -1;
	}

	if (iRet <= 0)
		return -1;

	return 0;
}

//********************************************//
//  Description:
//      Print configuring information string.
//********************************************//
void CUserConfig::PrintSelectMode()
{
	PrintBlank(3);
	PrintString(pStrUI[45]);
	PrintBlank(6);
	PrintString(pStrUI[46]);
	PrintBlank(6);
	PrintString(pStrUI[54]);
	PrintBlank(6);
	PrintString(pStrUI[47]);
	PrintBlank(6);
	PrintString(pStrUI[48]);
	PrintBlank(6);
	PrintString(pStrUI[5]);
}

//*******************************************//
//  Description:
//      User set the configuration parameter
//*******************************************//
void CUserConfig::ConfigSelectMode()
{
	char szInputBuf[INPUT_NUM_MAX] = {0};
	int nInputValue = 0;
	int	iRet = 0;

	while (1)
	{
		PrintSelectMode();
		memset(szInputBuf, 0, sizeof(szInputBuf));
		iRet = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1);
		if (iRet == 1)
		{
			nInputValue = 1;//Auto Mode
			break;
		}
		if (iRet == -1)
		{
			PrintWarning();
			continue;
		}

		nInputValue = atoi(szInputBuf);
		if ((nInputValue != 1) && (nInputValue != 2) && (nInputValue != 3) )
		{
			PrintWarning();
			continue;
		}

		if (nInputValue == 1)
		{
			m_ShutdownParam.AutoMode = AUTOMODE_ALL_USB_WITH_COM;//Auto Mode
			break;
		}
		else if (nInputValue == 2)
		{
			m_ShutdownParam.AutoMode = AUTOMODE_ALL_USB_WITHOUT_COM;//Only USB
			break;
		}
		else if (nInputValue == 3)
		{
			m_ShutdownParam.AutoMode = AUTOMODE_STATIC_COM;//Only COM
			PrintNewLine();
			ConfigComPort();
			break;
		}
	}
	PrintNewLine();
}

//*******************************************//
//  Description:
//      User set the configuration parameter
//*******************************************//
void CUserConfig::ConfigComPort()
{
	int nComCount = 0;
	int ComList[256];
	int nIndex = 0;
	char szBuf[64];
	char szInputBuf[INPUT_NUM_MAX];
	int nInputValue = 0;
	int nFlag = 0;
	int nRet;
	CUPS ups;

	memset(ComList, 0, sizeof(ComList));
	nComCount = ups.GetComPortCount(ComList);

	while (1)
	{
		PrintBlank(6);		
		PrintString(pStrUI[50]);
		for (nIndex = 1; nIndex<=nComCount; nIndex++)
		{
			PrintBlank(6);
			sprintf(szBuf, "%d. COM%d\n", nIndex, (ComList[nIndex-1]+1));
			PrintString(szBuf);
		}
		PrintBlank(6);
		sprintf(szBuf, pStrUI[53], (ComList[0]+1));
		PrintString(szBuf);

		PrintBlank(6);
		PrintString(pStrUI[51]);

		memset(szInputBuf, 0, sizeof(szInputBuf));
		nRet = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1);
		switch (nRet)
		{
			case 1:
				nInputValue = 1;
				nFlag = 1;
				break;
			case 0:
				nInputValue = atoi(szInputBuf);
				if (nInputValue == 0 || nInputValue > nComCount)
					PrintWarning();
				else
					nFlag = 1;
				break;
			default:
				PrintWarning();
				break;
		}

		if (nFlag == 1)
			break;
	}

	m_ShutdownParam.uPortType = 1;
	m_ShutdownParam.uComPort = nInputValue;
}

//********************************************//
//  Description:
//      Print configuring information string.
//********************************************//
void CUserConfig::PrintACFailDelayTime()
{
	PrintBlank(3);
	PrintString(pStrUI[6]);
	PrintBlank(6);
	PrintString(pStrUI[8]);
	PrintBlank(6);
	PrintString(pStrUI[9]);
	PrintBlank(6);
	PrintString(pStrUI[10]);
	PrintBlank(6);
	PrintString(pStrUI[11]);
	PrintBlank(6);
	PrintString(pStrUI[12]);
	PrintBlank(6);
	PrintString(pStrUI[13]);
	PrintBlank(6);
	PrintString(pStrUI[7]);
}

//*******************************************//
//  Description:
//      User set the configuration parameter
//*******************************************//
void CUserConfig::ConfigACFailDelayTime()
{
	char szInputBuf[INPUT_NUM_MAX] = {0};
	int nInputValue = 0;
	int	iRet = 0;

	while (1)
	{
		PrintACFailDelayTime();
		memset(szInputBuf, 0, sizeof(szInputBuf));
		iRet = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1);
		if (iRet == 1)
		{
			nInputValue = ACFAILDELAY_DEFAULT;
			break;
		}
		if (iRet == -1)
		{
			PrintWarning();
			continue;
		}
		nInputValue = atoi(szInputBuf);

		if ((nInputValue < 0) || (nInputValue > 36000))
		{
			PrintWarning();
			continue;
		}
		break;
	}

	m_ShutdownParam.uACFailDelayTime = nInputValue;

	PrintNewLine();
}

//********************************************//
//  Description:
//      Print configuring information string.
//********************************************//
void CUserConfig::PrintExternCmd()
{
	PrintBlank(3);
	PrintString(pStrUI[15]);
	PrintBlank(6);
	PrintString(pStrUI[9]);
	PrintBlank(6);
	PrintString(pStrUI[17]);
	PrintBlank(6);
	PrintString(pStrUI[18]);
	PrintBlank(6);
	PrintString(pStrUI[16]);
}

//*******************************************//
//  Description:
//      User set the configuration parameter
//*******************************************//
void CUserConfig::ConfigExternCmd()
{
	int rtn = 0;
	char  szInputCmd[INPUT_NUM_MAX] = {0};

	PrintExternCmd();

	memset(szInputCmd, 0, sizeof(szInputCmd));
	rtn = ReadStdInput(szInputCmd, INPUT_NUM_MAX-1);

	memcpy(m_ShutdownParam.szExternCmd, szInputCmd, sizeof(szInputCmd));

	PrintNewLine();

}

//********************************************//
//  Description:
//      Print configuring information string.
//********************************************//
void CUserConfig::PrintExCmdNeedTime()
{
	PrintBlank(3);
	PrintString(pStrUI[19]);
	PrintBlank(6);
	PrintString(pStrUI[21]);
	PrintBlank(6);
	PrintString(pStrUI[9]);
	PrintBlank(6);
	PrintString(pStrUI[22]);
	PrintBlank(6);
	PrintString(pStrUI[23]);
	PrintBlank(6);
	PrintString(pStrUI[24]);
	PrintBlank(6);
	PrintString(pStrUI[25]);
	PrintBlank(6);
	PrintString(pStrUI[20]);
}

//*******************************************//
//  Description:
//      User set the configuration parameter
//*******************************************//
void CUserConfig::ConfigExCmdNeedTime()
{
	char szInputBuf[INPUT_NUM_MAX] = {0};
	int nInputValue = 0;
	int	iRet = 0;

	while (1)
	{
		PrintExCmdNeedTime();
		memset(szInputBuf, 0, sizeof(szInputBuf));
		iRet = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1);
		if (iRet == 1)
		{
			nInputValue = EXCMDNEEDTIME_DEFAULT;
			break;
		}
		if (iRet == -1)
		{
			PrintWarning();
			continue;
		}
		nInputValue = atoi(szInputBuf);

		if ((nInputValue >= 0) && (nInputValue <= 600))
		{
			if ((nInputValue % 60) != 0)
			{
				PrintWarning();
				continue;
			}
		}
		else
		{
			PrintWarning();
			continue;
		}
		break;
	}

	m_ShutdownParam.uExCmdNeedTime = nInputValue;

	PrintNewLine();
}

//********************************************//
//  Description:
//      Print configuring information string.
//********************************************//
void CUserConfig::PrintShutdownNeedTime()
{
	PrintBlank(3);
	PrintString(pStrUI[26]);
	PrintBlank(6);
	PrintString(pStrUI[28]);
	PrintBlank(6);
	PrintString(pStrUI[9]);
	PrintBlank(6);
	PrintString(pStrUI[29]);
	PrintBlank(6);
	PrintString(pStrUI[30]);
	PrintBlank(6);
	PrintString(pStrUI[31]);
	PrintBlank(6);
	PrintString(pStrUI[32]);
	PrintBlank(6);
	PrintString(pStrUI[27]);
}

//*******************************************//
//  Description:
//      User set the configuration parameter
//*******************************************//
void CUserConfig::ConfigShutdownNeedTime()
{
	char szInputBuf[INPUT_NUM_MAX] = {0};
	int nInputValue = 0;
	int	iRet = 0;

	while (1)
	{
		PrintShutdownNeedTime();
		memset(szInputBuf, 0, sizeof(szInputBuf));
		iRet = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1);
		if (iRet == 1)
		{
			nInputValue = SDNEEDTIME_DEFAULT;
			break;
		}
		if (iRet == -1)
		{
			PrintWarning();
			continue;
		}
		nInputValue = atoi(szInputBuf);

		if ((nInputValue >= 0) && (nInputValue <= 600))
		{
			if ((nInputValue % 60) != 0)
			{
				PrintWarning();
				continue;
			}
		}
		else
		{
			PrintWarning();
			continue;
		}
		break;
	}

	m_ShutdownParam.uShutdownNeedTime = nInputValue;
	PrintNewLine();
}

//********************************************//
//  Description:
//      Print configuring information string.
//********************************************//
void CUserConfig::PrintSendMessage()
{
	PrintBlank(3);
	PrintString(pStrUI[33]);
	PrintBlank(6);
	PrintString(pStrUI[34]);
	PrintBlank(6);
	PrintString(pStrUI[35]);
	PrintBlank(6);
	PrintString(pStrUI[5]);
}

//*******************************************//
//  Description:
//      User set the configuration parameter
//*******************************************//
void CUserConfig::ConfigSendMessage()
{
	char szInputBuf[INPUT_NUM_MAX] = {0};
	int nInputValue = 0;
	int	iRet = 0;

	while (1)
	{
		PrintSendMessage();
		memset(szInputBuf, 0, sizeof(szInputBuf));
		iRet = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1);
		if (iRet == 1)
		{
			nInputValue = SENDMSG_DEFAULT + 1;
			break;
		}
		if (iRet == -1)
		{
			PrintWarning();
			continue;
		}
		nInputValue = atoi(szInputBuf);

		if ((nInputValue != 1) && (nInputValue != 2))
		{
			PrintWarning();
			continue;
		}

		break;
	}

	m_ShutdownParam.iSendMsg = nInputValue - 1;
	PrintNewLine();
}

void CUserConfig::PrintUpsCommMode()
{
	PrintBlank(3);
	PrintString(pStrUI[59]);
	PrintBlank(6);
	PrintString(pStrUI[60]);
	PrintBlank(6);
	PrintString(pStrUI[61]);
	PrintBlank(6);
	PrintString(pStrUI[5]);
}

void CUserConfig::ConfigUpsCommMode()
{
	char szInputBuf[INPUT_NUM_MAX] = {0};
	int nInputValue = 0;
	int	iRet = 0;

	while (1)
	{
		PrintUpsCommMode();
		memset(szInputBuf, 0, sizeof(szInputBuf));
		iRet = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1);
		if (iRet == 1)
		{
			nInputValue = UpsCommMode_DEFAULT + 1;
			break;
		}
		if (iRet == -1)
		{
			PrintWarning();
			continue;
		}
		nInputValue = atoi(szInputBuf);

		if ((nInputValue != 1) && (nInputValue != 2))
		{
			PrintWarning();
			continue;
		}
		break;
	}

	m_ShutdownParam.iUpsCommMode = nInputValue - 1;
	PrintNewLine();
}

void CUserConfig::PrintShutdownMode()
{
	PrintBlank(3);
	PrintString(pStrUI[63]);
	PrintBlank(6);
	PrintString(pStrUI[64]);
	PrintBlank(6);
	PrintString(pStrUI[65]);
	PrintBlank(6);
	PrintString(pStrUI[5]);
}

void CUserConfig::ConfigShutdownMode()
{
	char szInputBuf[INPUT_NUM_MAX] = {0};
	int nInputValue = 0;
	int iRet = 0;

	while (1)
	{
		PrintShutdownMode();
		memset(szInputBuf, 0, sizeof(szInputBuf));
		iRet = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1);
		if (iRet == 1)
		{
			nInputValue = ShutdownMode_DEFAULT + 1;
			break;
		}
		if (iRet == -1)
		{
			PrintWarning();
			continue;
		}
		nInputValue = atoi(szInputBuf);

		if (2 == nInputValue)
		{
			int dev;
			char buf_state [50] = {0};
			int n;
			dev = open("/sys/power/state", O_RDONLY); 
			if (dev == NULL)
			{
				printf("Your system does not support the hibernation, please choice Shutdow mode!\n");
				continue;
			}
			if ((n = read(dev, buf_state, 50)) > 0)
			{
				char *p = strstr(buf_state, "disk");
#ifdef Cent47
				p = NULL;
#endif
				if (p == NULL)
				{
					printf("Your system does not support the hibernation, please choice Shutdow mode!\n");
					continue;
				}
			}
			else
			{
				printf("Your system does not support the hibernation, please choice Shutdow mode!\n");
				continue;
			}
			close(dev);
		}

		if ((nInputValue != 1) && (nInputValue != 2))
		{
			PrintWarning();
			continue;
		}
		break;
	}

	m_ShutdownParam.iShutdownMode= nInputValue - 1;
	PrintNewLine();
}

void CUserConfig::PrintKVMMode()
{
	PrintBlank(3);
	PrintString(pStrUI[67]);
	PrintBlank(6);
	PrintString(pStrUI[68]);
	PrintBlank(6);
	PrintString(pStrUI[69]);
	PrintBlank(6);
	PrintString(pStrUI[5]);
}

void CUserConfig::ConfigKVMMode()
{
	char szInputBuf[INPUT_NUM_MAX] = {0};
	int nInputValue = 0;
	int iRet = 0;

	while (1)
	{
		PrintKVMMode();
		memset(szInputBuf, 0, sizeof(szInputBuf));
		iRet = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1);
		if (iRet == 1)
		{
			nInputValue = KVMMode_DEFAULT ; //modify 2011824
			break;
		}
		if (iRet == -1)
		{
			PrintWarning();
			continue;
		}
		nInputValue = atoi(szInputBuf);

		if ((nInputValue != 1) && (nInputValue != 2))
		{
			PrintWarning();
			continue;
		}
		break;
	}

	m_ShutdownParam.iKVMMode = 2 - nInputValue ;  //modify 2011824
	PrintNewLine();
}
// allen20110916
void CUserConfig::PrintGuestOSMode()
{
	PrintBlank(3);
	PrintString(pStrUI[71]);
	PrintBlank(6);
	PrintString(pStrUI[72]);
	PrintBlank(6);
	PrintString(pStrUI[73]);
	PrintBlank(6);
	PrintString(pStrUI[5]);
}
//allen20110916
void CUserConfig::ConfigGuestOSMode()
{
	char szInputBuf[INPUT_NUM_MAX] = {0};
	int nInputValue = 0;
	int iRet = 0;

	while (1)
	{
		PrintGuestOSMode();
		memset(szInputBuf, 0, sizeof(szInputBuf));
		iRet = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1);
		if (iRet == 1)
		{
			nInputValue = GuestOSMode_DEFAULT + 1;//0 shutdown  
			break;
		}
		if (iRet == -1)
		{
			PrintWarning();
			continue;
		}
		nInputValue = atoi(szInputBuf);

		if ((nInputValue != 1) && (nInputValue != 2))
		{
			PrintWarning();
			continue;
		}
		break;
	}

	m_ShutdownParam.iGuestOSMode =  nInputValue - 1; 
	PrintNewLine();
}

//********************************************//
//  Description:
//      Print configuring information string.
//********************************************//
void CUserConfig::PrintReconfirmInfo()
{
	char szBuf[1024] = {0};
	char szMsgSetting[512] = {0};
	// added for Ups Communication Mode
	char szUpsCommMode[512] = {0};
	// added for support Hibernation
	char szShutdownMode[512] = {0};
	// added for support KVM
	char szKVMMode[512] = {0};
	// added for Guest OS 
	char szGuestOSMode[512] = {0};
	int nIndex = 0;

	ClearScreen();
	PrintString(pStrUI[1]);
	sprintf(szBuf, pStrUI[36], m_szHostName);
	PrintString(szBuf);
	PrintString(pStrUI[1]);

	if (m_ShutdownParam.AutoMode == AUTOMODE_ALL_USB_WITH_COM)
	{
		strcpy(szMsgSetting, pStrUI[55]);
		sprintf(szBuf, "%d.%s%s\n", ++nIndex, pStrUI[49], szMsgSetting);
		PrintString(szBuf);
	}
	else if (m_ShutdownParam.AutoMode == AUTOMODE_ALL_USB_WITHOUT_COM)
	{
		strcpy(szMsgSetting, pStrUI[56]);
		sprintf(szBuf, "%d.%s%s\n", ++nIndex, pStrUI[49], szMsgSetting);
		PrintString(szBuf);
	}
	else if (m_ShutdownParam.AutoMode == AUTOMODE_STATIC_COM)
	{
		strcpy(szMsgSetting, pStrUI[57]);
		sprintf(szBuf, "%d.%s%s\n", ++nIndex, pStrUI[49], szMsgSetting);
		PrintString(szBuf);
		sprintf(szBuf, "%d.%sCOM%d\n", ++nIndex, pStrUI[52], m_ShutdownParam.uComPort);//check num
		PrintString(szBuf);
	}
	//add Ups Communication Mode
	if (m_ShutdownParam.iUpsCommMode == 1)
		strcpy(szUpsCommMode, "HID");
	else
		strcpy(szUpsCommMode, "Libusb");

	sprintf(szBuf, "%d.%s%s\n", ++nIndex, pStrUI[62], szUpsCommMode);
	PrintString(szBuf);

	sprintf(szBuf, "%d.%s%d\n", ++nIndex, pStrUI[37], m_ShutdownParam.uACFailDelayTime);
	PrintString(szBuf);
	sprintf(szBuf, "%d.%s%s\n", ++nIndex, pStrUI[38], m_ShutdownParam.szExternCmd);
	PrintString(szBuf);

	memset(&szBuf, 0, sizeof(szBuf));

	sprintf(szBuf, "%d.%s%d\n", ++nIndex, pStrUI[39], m_ShutdownParam.uExCmdNeedTime);
	PrintString(szBuf);
	sprintf(szBuf, "%d.%s%d\n", ++nIndex, pStrUI[40], m_ShutdownParam.uShutdownNeedTime);
	PrintString(szBuf);
	if (m_ShutdownParam.iSendMsg == 1)
		strcpy(szMsgSetting, "Enable");
	else
		strcpy(szMsgSetting, "Disable");

	sprintf(szBuf, "%d.%s%s\n", ++nIndex, pStrUI[41], szMsgSetting);
	PrintString(szBuf);
	//add for support Hibernation
	if (m_ShutdownParam.iShutdownMode == 1)
		strcpy(szShutdownMode, "Hibernate");
	else
		strcpy(szShutdownMode, "Shutdown");

	sprintf(szBuf, "%d.%s%s\n", ++nIndex, pStrUI[66], szShutdownMode);
	PrintString(szBuf);

	//add for support KVM
	if (m_ShutdownParam.iShutdownMode == 0)
	{
		if (JudgeSySSupportKVM())
		{
			if (m_ShutdownParam.iKVMMode == 1)
				strcpy(szKVMMode, "Shutdown by Linux System");
			else
				strcpy(szKVMMode, "Shutdown by Simple Shutdown");

			sprintf(szBuf, "%d.%s%s\n", ++nIndex, pStrUI[70], szKVMMode);
			PrintString(szBuf);
		}
	}
	//add for support GuestOSMode  allen20110916
	memset(&szBuf, 0, sizeof(szBuf));
	if ((m_ShutdownParam.iShutdownMode == 0) && !m_ShutdownParam.iKVMMode)
	{
		if (!m_ShutdownParam.iGuestOSMode)
			strcpy(szGuestOSMode, "Shutdown");
		else
			strcpy(szGuestOSMode, "Suspend");

		sprintf(szBuf, "%d.%s%s\n", ++nIndex, pStrUI[74], szGuestOSMode);
		PrintString(szBuf);
	}

	PrintString(pStrUI[1]);
	PrintString(pStrUI[42]);
	PrintString(pStrUI[5]);
}

//**************************************//
//  Return value:
//      Return the number user input.
//  Description:
//      Let user confirm the configuration.
//**************************************//
int CUserConfig::ConfigReconfirm()
{
	char szInputBuf[INPUT_NUM_MAX] = {0};
	int nInputValue = 0;
	int irtn;

	while (1)
	{
		PrintReconfirmInfo();
		memset(szInputBuf, 0, sizeof(szInputBuf));
		irtn = ReadStdInput(szInputBuf, INPUT_NUM_MAX-1);

		if (irtn == -1)
		{
			PrintWarning();
			memset(szInputBuf, 0, sizeof(szInputBuf));
			continue;
		}

		nInputValue = atoi(szInputBuf);

		if ((nInputValue != 1) && (nInputValue != 2))
		{
			PrintWarning();
			continue;
		}

		break;
	}

	return nInputValue;
}

//***********************************************************************************************************************//
//  Parameters:
//      shutdownParam
//          [out] Store configuration info to the structure variable shutdownParam.
//  Return value:
//      Return 0.
//  Description:
//      Implement the menu application config.sh, and the shutdown parameters user setted are in variable shutdownParam.
//***********************************************************************************************************************//
int CUserConfig::ConfigShutdownParam(SHUTDOWNPARAM &shutdownParam)
{
	int iRet = 0;

	do
	{
		ClearScreen();
		PrintConfigTitle();
		ConfigSelectMode();
		ConfigUpsCommMode();	//modify 20110824
		ConfigACFailDelayTime();
		ConfigExternCmd();
		ConfigExCmdNeedTime();
		ConfigShutdownNeedTime();
		ConfigSendMessage();		
		ConfigShutdownMode();
		if (m_ShutdownParam.iShutdownMode == 0)
		{
			if (JudgeSySSupportKVM() == 1)
				ConfigKVMMode();
			else
			{
				m_ShutdownParam.iKVMMode = 1; 
				//sys support KVM . if sys did`t support kvm ,will do nothing
			}
			if (!m_ShutdownParam.iKVMMode)    //allen20110916   select simple shutdown
				ConfigGuestOSMode();
		}

		iRet = ConfigReconfirm();
		if (iRet != 1)
		{
			InitShutdownParam();
			continue;
		}
		break;
	}while(1);

	shutdownParam = m_ShutdownParam;
	return 0;
}

int CUserConfig::JudgeSySSupportKVM()
{
	int gust_val;
	int fd[2];
	int pid;
	char line[512] = {0};

	if (pipe(fd) < 0) //call system call (pipe) to create a pipe
	{
		perror("pipe");
		exit(1);
	}
	//use fork to create a child process
    //child process will wrtie data to pipe, and parent will read data from pipe
    //child process
	if ((pid = fork()) < 0)
	{
		perror("fork");
		exit(1);
	}
	if (pid > 0)
	{
		close(fd[1]);
		wait(NULL);
		gust_val = read(fd[0], line, 512); 
		if (gust_val == 0)
		{
			close(fd[0]);
			return 0;
		}
		else if (strncmp(line, "kvm", 3) == 0)
		{
		    close(fd[0]);
			return 1;
		}
		close(fd[0]);
	}
	else
	{
		close(fd[0]);
		int tmp = dup(1); //backup stdout

		dup2(fd[1], 1); //stdout redirectory 
		system("lsmod | grep -w kvm");
		dup2(tmp, 1);//resume
		close(fd[1]);
		_exit(1);
	}

	return 0;
}

int main(int argc, char **argv)
{
	CPublic pub;
	CConfigFile cfg;
	SHUTDOWNPARAM shutdownParam;
	CUserConfig	userCfg;
	int	iLockfd = 0;

	if (argc >= 2)
		printf("         paramter error!\n");

	if (pub.CheckLock(LOCKFILE_CONFIG))
		return 0;

	iLockfd = pub.SetLock(LOCKFILE_CONFIG);

	if (pub.CheckLock(LOCKFILE_AGENT))
	{
		printf("Warning: The Simple Shutdown Service is running, please stop it firstly.\n");
		printf("         To stop the Simple Shutdown Service please run command:\n");
		printf("         /usr/lib/ssd/master/AgentManager\n");

		pub.UnsetLock(iLockfd, LOCKFILE_CONFIG);
		unlink(LOCKFILE_CONFIG);
		return -1;
	}

	cfg.SetDefaultShutdownParam(&shutdownParam);	
	if (userCfg.ConfigShutdownParam(shutdownParam) == 0)
	{
		cfg.SetShutdownParameter(shutdownParam);
		pub.System(PROCESS_SERVICE);
	}
	pub.UnsetLock(iLockfd, LOCKFILE_CONFIG);
	unlink(LOCKFILE_CONFIG);
	return 0;
}

