#include "Event.h"

CEvent::CEvent()
{
}

CEvent::~CEvent()
{

}

//**********************************************************************************************************//
//  Parameters:
//      pszMsg
//          [in] Message to broadcast.
//  Description:
//      Open a pipe with function popen(), and use shell command "wall" to send message to the login users.
//**********************************************************************************************************//
void CEvent::BroadcastMessage(char *pszMsg)
{
	FILE *fp = NULL;
	char szCmd[256] = {0};

	if (pszMsg == NULL)
		return;
	
	strcpy(szCmd, "wall");
	
	if ((fp = popen(szCmd, "w")) != NULL)
	{
		fprintf(fp, "%s", pszMsg);
		pclose(fp);
	}
}

//***********************************************************************************//
//  Parameters:
//      pszData
//          [in] Recorded log message.
//  Return value:
//      Return 1, succeed; otherwise failed.
//  Description:
//      Write message into the local file, and if the log is more than 2000 pieces,
//      the former logs will be removed, and the latest logs will be kept.
//***********************************************************************************//
int CEvent::WriteLogFile(char * pszData)
{
	char szBuf[MAX_LINELENGTH];
	DWORD dwRowCount = 0;
	time_t tNow = 0;
	struct tm *ptm = NULL;
	FILE* fp = NULL;
	FILE* fpTmp = NULL;
	char FilePath[MAX_FILE];
	char szLogBuf[MAX_LINELENGTH];

	tNow = time(NULL);
	ptm = localtime(&tNow);

	if (!ptm)
		return -1;

	// Record event log
	memset(szBuf, 0, sizeof(szBuf));

	sprintf(szBuf,
		"%04d/%02d/%02d %02d:%02d:%02d %s\n",
		ptm->tm_year+1900,
		ptm->tm_mon+1,
		ptm->tm_mday,
		ptm->tm_hour,
		ptm->tm_min,
		ptm->tm_sec,
		pszData);

	memcpy(&FilePath, PATH_EVENTLOG, MAX_FILE);
	fp = fopen(FilePath,"a+t");
	if (fp == NULL)
		return -1;

	memset(szLogBuf, 0, sizeof(szLogBuf));
	rewind(fp);
	
	while (fgets(szLogBuf, MAX_LINELENGTH, fp))
	{
		dwRowCount++;
	}		
	if (dwRowCount < MAX_EVENTLOG_COUNT)
	{
		fwrite(szBuf, strlen(szBuf), 1, fp);
		fflush(fp);
		fclose(fp);
	}
	else
	{
		fseek(fp,0,SEEK_SET);
		
		fpTmp = fopen(TEMP_FILE, "w");
		if (fpTmp == NULL)
			return -1;

		while (dwRowCount >= MAX_EVENTLOG_COUNT)
		{
			fgets(szLogBuf,MAX_LINELENGTH, fp);
			dwRowCount--;
		}
		while (fgets(szLogBuf, MAX_LINELENGTH, fp))
		{
			fwrite(szLogBuf,strlen(szLogBuf),1,fpTmp);
		}
		fwrite(szBuf, strlen(szBuf), 1, fpTmp);
		
		fclose(fp);
		fflush(fpTmp);
		fclose(fpTmp);
		if (remove(FilePath) != 0)
			return 0;

		if (rename(TEMP_FILE,FilePath) != 0)
			return 0;
	}
	
	return 1;
}

//**************************************************************************************//
//  Parameters:
//      pszData
//          [in] Recorded log message.
//  Return value:
//      Return 1, succeed; otherwise failed.
//  Description:
//      Support syslog, and invoke function WriteLogFile() to record log in local file.
//**************************************************************************************//
int CEvent::WriteEventLog(char * pszData)
{
	openlog("SimpleShutdown", LOG_PID, LOG_DAEMON);

	syslog(LOG_NOTICE, pszData);
	closelog();
	
	return WriteLogFile(pszData);
}

