#include "Public.h"

CPublic::CPublic()
{
}

CPublic::~CPublic()
{
}

//********************************************************************//
//  Parameters:
//      pszLockFile
//          [in] File path.
//  Return value:
//      Return the file descriptor, if succeed; otherwise return -1.
//  Description:
//      Set a file lock.
//********************************************************************//
int CPublic::SetLock(char *pszLockFile)
{
	int     ifileid;
 	struct 	flock lock;
 	char 	szBuf[256];
 	
	if ((ifileid = open(pszLockFile, O_RDWR|O_CREAT, FILE_MODE)) == -1)
    	return -1;

	DelayInUsec(0, 10000);

	lock.l_type = F_WRLCK;
	lock.l_whence = SEEK_SET;
	lock.l_start = 0;
	lock.l_len = 0;
  	
	if (fcntl(ifileid, F_SETLK, &lock) == -1)
		return -1;
	
	memset(szBuf, 0, sizeof(szBuf));
	snprintf(szBuf, sizeof(szBuf), "%ld\n", (long)getpid());
	ftruncate(ifileid, 0);
	write(ifileid, szBuf, strlen(szBuf));

    return ifileid;
}

//**************************************************//
//  Parameters:
//      pszFile
//          [in] File path.
//  Return value:
//      0, file not locked; 1, file locked.
//  Description:
//      Check if the file (pszLockFile) is locked.
//**************************************************//
int CPublic::CheckLock(char *pszLockFile)
{
	int ifileid;
 	struct flock lock;
	
	if ((ifileid = open(pszLockFile, O_RDWR, FILE_MODE)) == -1)
		return 0;
	
	DelayInUsec(0, 10000);
  
	lock.l_type = F_WRLCK;
	lock.l_whence = SEEK_SET;
	lock.l_start = 0;
	lock.l_len = 0;
	
	if ((fcntl(ifileid, F_GETLK, &lock)) == -1)
	{
		close(ifileid);
		return 1;
    }
    
	if (lock.l_type == F_UNLCK)
	{
		close(ifileid);
		return 0;
    }

	close(ifileid);
	return 1;
}

//******************************************************//
//  Parameters:
//      ifd
//          [in] It is returned by SetLock().
//      pszLockFile
//          [in] File path.
//  Return value:
//      If return 0, failed, if return 1, succeed.
//  Description:
//      If the file (pszLockFile) is locked, unlock it.
//******************************************************//
int CPublic::UnsetLock(int ifileid, char *pszLockFile)
{
 	struct flock lock;
	
	if (!CheckLock(pszLockFile))
		return 1;

	if (ifileid <= 0)
    	return 1;
	
	lock.l_type = F_UNLCK;
	lock.l_whence = SEEK_SET;
	lock.l_start = 0;
	lock.l_len = 0;

	if ((fcntl(ifileid, F_SETLK, &lock)) == -1)
		return 0;

	close(ifileid);
	return 1;
}

void CPublic::DelayInUsec(int sec, int usecond)
{
	fd_set fdpool;
	struct timeval  tm;

	FD_ZERO(&fdpool);
  
	tm.tv_sec  = sec;
	tm.tv_usec = usecond;
	select(10, &fdpool, NULL, NULL, &tm);
}

//*************************************************//
//  Return value:
//      If succeed, return 0; if failed, return 0.
//  Description:
//      To create a daemon service.
//*************************************************//
int CPublic::CreateDaemon()
{
	int iPid = 0;
	int i = 0;

	iPid = fork();	
	if (iPid > 0)
		exit(0);
	else if (iPid < 0)
		exit(1);
	
	setsid();
	
	iPid = fork();
	if (iPid > 0)
		exit(0);
	else if (iPid < 0)
		exit(1);

	for (i = 0; i < 128; i++)
	{
		close(i);
	}

    i = open("/dev/null", O_RDWR);
    if (i < 0)
    {
        _exit(1);
        return(-1);
    }
    dup2(i, 1);
    dup2(i, 2);
	
	for (i = 0; i < 32; i++)
	{
		if ((i != SIGCHLD) && (i != SIGUSR1) && (i != SIGUSR2))
			signal(i, SIG_IGN);
	}
	
	chdir("/");
	umask(0);
	
	return 0; 
}

//**************************************************************************//
//  Parameters:
//      pszLockFile
//          [in] File path.
//  Return value:
//      1, service is running; 0, service is not running.
//  Description:
//      Invoke CheckLock() to see if the related daemon service is running.
//**************************************************************************//
int CPublic::IsServiceActive(char *pszLockFile)
{
	return CheckLock(pszLockFile);
}

//***********************************************//
//  Parameters:
//      pszCmd
//          [in] The system command to execute.
//  Return value:
//      If pszCmd is NULL, return -1.
//  Description:
//      Fork a thread to execute system command.
//***********************************************//
int CPublic::System(char *pszCmd)
{
	int i;
	
	if (pszCmd == NULL)
		return -1;
	
	for (i = 0; i < 32; i++)
	{
		signal(i,SIG_IGN);
	}
	
	if (fork() == 0)
    {
 		for (i = 0; i < 128; i++)
 		{
  			close(i);
 		}
 
        execlp(pszCmd, NULL, NULL);
		_exit(0);
    }
    
	return 1;
}

//****************************************************************************************************//
//  Parameters:
//      pchFifoName
//          [in] FIFO name.
//      nWrite
//          [in] FIFO permission mode.
//  Return value:
//      If succeed, return the FIFO descriptor; if -1, create FIFO failed, if -2, open FIFO failed.
//  Description:
//      Create the FIFO, and opened it.
//****************************************************************************************************//
int CPublic::OpenFifo(char *pchFifoName, int nWrite)
{
	int nFd, nRet, nFlag;

	if ((nRet = mkfifo(pchFifoName,MYFILE_MODE)) == -1)
	{
		if (errno != EEXIST)
			return -1;
	}

	if (nWrite == 1)
		nFlag = O_WRONLY;
	else
		nFlag = O_RDONLY;

	if ((nFd = open(pchFifoName,nFlag|O_NONBLOCK)) == -1)
		return -2;

	return nFd;
}

//***************************************************//
//  Parameters:
//      writefifofd
//          [in] FIFO file descriptor.
//      pipeData
//          [in] Data to be send by FIFO.
//  Return value:
//      If 1, succeed; if 0, failed.
//  Description:
//      Send the struct data by FIFO (writefifiofd).
//***************************************************//
int CPublic::SendFIFOData(int writefifofd, PPIPE_DATA pipeData)
{
	int nCount = 0;
	char szData[MAX_FIFODATA_LEN];
	DWORD dwHeadLen = 0;
	DWORD dwLength = 0;
	FIFO_DATA_HEAD fdh;
	struct timeval tv;

	dwHeadLen = sizeof(FIFO_DATA_HEAD);
	dwLength = sizeof(PIPE_DATA);
	
	memset(&fdh, 0, sizeof(fdh));
	memset(szData, 0, sizeof(szData));
	fdh.dwCmdId = pipeData->nCmdType;
	fdh.dwDataLen = dwLength;
	memcpy(szData, &fdh, dwHeadLen);
	memcpy(szData+dwHeadLen, pipeData, dwLength);

	tv.tv_sec = 0;
	tv.tv_usec = 500000;

	while (nCount++ < 5)
	{
		if (write(writefifofd, szData, dwLength+dwHeadLen) != -1)
			break;

		DelayInUsec(0, 200000);
	}
	
	if (nCount <= 5)
		return 1;

	return 0;
}

//*****************************************************//
//  Parameters:
//      readfifofd
//          [in] FIFO file descriptor.
//      pipeData
//          [out] Storage location of the read data.
//  Return value:
//      If read struct data succeed, return 1.
//  Description:
//      Read struct data by FIFO (readfifofd).
//*****************************************************//
int CPublic::ReadFIFOData(int readfifofd, PPIPE_DATA pipeData)
{
	int 	nRet = 0;
	fd_set 	readset;
	struct 	timeval tv;

	int 	iHeadLen;
	FIFO_DATA_HEAD FifoHead;
	memset(&FifoHead, 0, sizeof(FIFO_DATA_HEAD));

	iHeadLen = sizeof(FIFO_DATA_HEAD);
	
	tv.tv_sec = 0;
	tv.tv_usec = 500000;
	FD_ZERO(&readset);
	FD_SET(readfifofd,&readset);
	
	if ((nRet = select(readfifofd+1, &readset, NULL, NULL, &tv)) > 0)
	{
		if (FD_ISSET(readfifofd,&readset))
		{
			nRet = read(readfifofd, &FifoHead, iHeadLen);	
			if (nRet == iHeadLen)
			{
				if (FifoHead.dwDataLen > 0)
				{
					nRet = read(readfifofd, pipeData, FifoHead.dwDataLen);
			
					if (nRet == (int) FifoHead.dwDataLen)
							return 1;
				}
			}	
		}
	}	
	return  0;
}

//**************************************//
//  Parameters:
//      nFifoId
//          [in] FIFO file descriptor.
//  Return value:
//      Return 1, if succeed.
//  Description:
//      Close the FIFO.
//**************************************//
int CPublic::CloseFifo(int nFifoId)
{
	return close(nFifoId);
}

//********************************************//
//  Parameters:
//      pchFifoName
//          [in] FIFO name.
//  Return value:
//      Return 0, succeed, otherwise filed.
//  Description:
//      Remove the FIFO.
//********************************************//
int CPublic::UnlinkFifo(char *pchFifoName)
{
	return unlink(pchFifoName);
}

int CPublic::GetSize(int iIPCKey)
{
	int idSM, shmid;
	struct shmid_ds shmseg;

	if ((idSM = shmget(iIPCKey,0,IPC_CREAT|700)) == -1)
		return -1;

	shmid = shmctl(idSM, IPC_STAT, &shmseg);	
	if ( shmid < 0 )
		return -2;

	return shmseg.shm_segsz;
}

int CPublic::GetIPCkey(int key)
{
	return (ftok(SHM_FILE,key));
}

int CPublic::CheckIsExist(int iIPCKey)
{
	int idSM, iIPCKeyVal;

	iIPCKeyVal = GetIPCkey(iIPCKey);
	if ((idSM = shmget(iIPCKeyVal, 0,SM_OPEN|IPC_EXCL)) == -1)
	{
		if (errno != EEXIST)
			return -1;
		else
			return 1;	
	}
	return 0;
}

//********************************************************************************//
//  Parameters:
//      iIPCKey
//          [in] Used to generate the semaphore related to the share memeory.
//      size_val
//          [in] Size of share memory.
//  Return value:
//      Failed, return -2; return 0, open failed.
//  Description:
//      Create a share memory.
//********************************************************************************//
int CPublic::CreateSM(int iIPCKey, int size_val)
{
	int idSM;
	int iIPCKeyVal;

	iIPCKeyVal = GetIPCkey(iIPCKey);

	if ((idSM = shmget(iIPCKeyVal,size_val,SM_OPEN|IPC_EXCL)) == -1)
		return -2;

	return 0;
}

//***********************************************************//
//  Parameters:
//      iIPCKey
//          [in] The semaphore related to the share memory.
//  Return value:
//      If return 0, succeed, otherwise failed.
//  Description:
//      Remove the share memory.
//***********************************************************//
int CPublic::DestorySM(int iIPCKey)
{
	int ret, idSM, iIPCKeyVal;

	WaitSem(iIPCKey);

	iIPCKeyVal = GetIPCkey(iIPCKey);

	if ((ret = CheckIsExist(iIPCKey)) == -1)
	{
		PostSem(iIPCKey);
		return -1;
	}

	if (ret)
	{
		if ((idSM = shmget(iIPCKeyVal,0,SM_OPEN)) == -1)
		{
			PostSem(iIPCKey);
			return -2;
		}
		shmctl(idSM,IPC_RMID,0);
	}

	PostSem(iIPCKey);
	return 0;
}

//********************************************************************************//
//  Parameters:
//      iIPCKey
//          [in] The semaphore related to the share memory.
//      ilen
//          [out] The size of the share memory data.
//  Return value:
//      Return storage location of the data from share memory, if NULL, failed.
//  Description:
//      Get data from share memory.
//********************************************************************************//
void* CPublic::GetSMData(int iIPCKey,int* ilen)
{
	int idSM, size_val, iIPCKeyVal;
	char buf[10000];

	void * pSM = NULL;
	void * param = NULL;

	WaitSem(iIPCKey);

	if (!CheckIsExist(iIPCKey))
	{
		PostSem(iIPCKey);
		return NULL;
	}

	iIPCKeyVal = GetIPCkey(iIPCKey);

	size_val = GetSize(iIPCKeyVal);
	if (size_val < 0)
	{
		PostSem(iIPCKey);
		*ilen = size_val;
		return NULL;
	}

	memset(buf, 0, 10000);
	sprintf(buf, "GetSMData[%d]", size_val);

	if ((idSM = shmget(iIPCKeyVal,0,SM_OPEN)) == -1)
	{   
		PostSem(iIPCKey);
		*ilen = -1; 
		return NULL;
	}
	if ((pSM = shmat(idSM,0,0)) == (void *) -1)
	{
		PostSem(iIPCKey);
		*ilen = -2;
		return NULL;
	}

	param = malloc(size_val);
	if (param == NULL)
	{
		PostSem(iIPCKey);
		*ilen = -3;
		return NULL;  
	}

	memset(param, 0, size_val);
	memcpy((void *)param, pSM, size_val);

	shmdt(pSM);
	PostSem(iIPCKey);

	*ilen = size_val;
	return(param);
}

//*************************************************************//
//  Parameters:
//      iIPCKey
//          [in] The semaphore related to the share memory.
//      param
//          [in] Data store to share memory.
//      paramlen
//          [in] Length of the data.
//  Return value:
//      If succeed, return paramlen;
//      if return -1, share memory not exist;
//      if return -2, open share memory failed;
//      if return -3, set data to share memory failed.
//  Description:
//      Set data to the share memory.
//*************************************************************//
int CPublic::SetSMData(int iIPCKey, void * param, int paramlen)
{
	int  idSM, iIPCKeyVal;
	void * pSM;

	WaitSem(iIPCKey);

	if (!CheckIsExist(iIPCKey))
	{
		PostSem(iIPCKey);
		return -1;
	}

	iIPCKeyVal = GetIPCkey(iIPCKey);

	if ((idSM = shmget(iIPCKeyVal,0,SM_OPEN)) == -1)
	{     
		PostSem(iIPCKey); 
		return -2;
	}
	if ((pSM = shmat(idSM,0,0)) == (void *) -1)
	{
		PostSem(iIPCKey);
		return -3;
	}

	memcpy(pSM, (void *)param, paramlen);
	shmdt(pSM);
	PostSem(iIPCKey);
	DelayInUsec(0, 100000);
	return(paramlen);
}

//**********************************************//
//  Return value:
//      Return the semaphore set identifier.
//  Description:
//      Initilaize the semaphore.
//**********************************************//
int CPublic::InitSems()
{
	int idSem, i, iIPCKeyVal;

	iIPCKeyVal = GetIPCkey(IPCKEY_SEMS);
	
	if ((idSem = semget(iIPCKeyVal,SEM_MAX,SEM_OPEN|IPC_EXCL|SEM_OPS)) < 0)
	{ 
		if (errno == EEXIST)
		{
			idSem =  semget(iIPCKeyVal,0,SEM_OPEN|SEM_OPS);
			semctl(idSem,0,IPC_RMID);
			idSem = semget(iIPCKeyVal,SEM_MAX,SEM_OPEN|IPC_EXCL|SEM_OPS);
		}
	}

	for (i = 0;i < SEM_MAX ; i++)
	{
		semctl(idSem,i,SETVAL,1);
	}

	return idSem;
}

//****************************************************************//
//  Return value:
//      Return the semaphore set identifier; if failed return -1.
//  Description:
//      Get the semaphore set identifier.
//****************************************************************//
int CPublic::GetSemsID()
{
	int idSem, i, iIPCKeyVal;

	iIPCKeyVal = GetIPCkey(IPCKEY_SEMS);

	if ((idSem = semget(iIPCKeyVal,SEM_MAX,SEM_OPEN|IPC_EXCL|SEM_OPS)) < 0)
	{ 
		if (errno == EEXIST)
		{
			idSem = semget(iIPCKeyVal,0,SEM_OPEN|SEM_OPS);
			return idSem;
		}
		else 
		{
			printf("\nSet semaphere error %d \n",errno);
			return -1;
		}
	}

	for (i = 0;i < SEM_MAX ; i++)
	{
		semctl(idSem,i,SETVAL,1);
	}

	return idSem;
}

int CPublic::CreateSems()
{
	int idSem, i, iIPCKeyVal;
	int bInit = 0;
	struct sembuf SemOp;
	struct semid_ds buf;

	iIPCKeyVal = GetIPCkey(IPCKEY_SEMS);

	if ((idSem = semget(iIPCKeyVal,SEM_MAX,SEM_OPEN|IPC_EXCL|SEM_OPS)) >= 0)
	{
		for (i = 0; i < SEM_MAX; i++)
		{
			semctl(idSem,i,SETVAL,1);
		}
	}
	else if(errno == EEXIST)
    {
        idSem =  semget(iIPCKeyVal,SEM_MAX,SEM_OPEN|SEM_OPS);
        for (i = 0; i < 10; i++)
        {
            semctl(idSem,0,IPC_STAT,&buf);

            if (buf.sem_otime != 0)
            {
                bInit = 1;
                break;
            }
        }
        if (!bInit)
            sleep(1);
    }

    if (!bInit)
    {
        for (i = 0; i < SEM_MAX ; i++)
        {
            SemOp.sem_num = i;
            SemOp.sem_op  = 1;
            SemOp.sem_flg = SEM_UNDO;
            semop(idSem,&SemOp,1);
        }
    }

    return idSem;
}

//***************************************************//
//  Parameters:
//      index
//          [in] The semaphore index.
//  Return value:
//      If succeed, return 0; if failed, return 0.
//  Description:
//      To check if the semaphore is allowed to use.
//***************************************************//
int CPublic::WaitSem(int index)
{
	struct sembuf SemOp;
	int idSem = GetSemsID();

	SemOp.sem_num = index;
	SemOp.sem_op  = -1;
	SemOp.sem_flg = SEM_UNDO;

	return(semop(idSem,&SemOp,1));
}

//******************************************************//
//  Parameters:
//      index
//          [in] The semaphore index.
//  Return value:
//      If succeed, return 0; if failed, return 0.
//  Description:
//      After operation, do recover for the semaphore.
//******************************************************//
int CPublic::PostSem(int index)
{
	struct sembuf SemOp;
	int idSem = GetSemsID();

	SemOp.sem_num = index;
	SemOp.sem_op  = 1;
	SemOp.sem_flg = SEM_UNDO;

	return(semop(idSem,&SemOp,1));
}

int CPublic::WaitSemX(int idSem,int index,short flag)
{
	struct sembuf SemOp;
	SemOp.sem_num = index;
	SemOp.sem_op  = -1;
	SemOp.sem_flg = flag;
	return(semop(idSem,&SemOp,1));
}

int CPublic::PostSemX(int idSem,int index,short flag)
{
	struct sembuf SemOp;
	SemOp.sem_num = index;
	SemOp.sem_op  = 1;
	SemOp.sem_flg = flag;
	return(semop(idSem,&SemOp,1));
}

void CPublic::DestorySem(int idSem)
{
	int iIPCKeyVal;

	sleep(1);

	iIPCKeyVal = GetIPCkey(idSem);
	iIPCKeyVal =  semget(iIPCKeyVal,0,SEM_OPEN|SEM_OPS);
	semctl(iIPCKeyVal, 0, IPC_RMID);
}

#if 1
/*
 * Create a server endpoint of a connection.
 * Return fd if all ok, <0 on error.
 */
int CPublic::serv_create()
{
	int listenfd;
	struct sockaddr_in	 servaddr;
	int one = 1;
	
	/* create a UNIX domain stream socket */
	if ((listenfd= Socket(AF_INET, SOCK_STREAM, 0))  < 0)
		return -1;

	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family      = AF_INET;
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servaddr.sin_port        = htons(SERV_PORT);

	setsockopt(listenfd, SOL_SOCKET, SO_REUSEADDR, (char *)&one, sizeof (one));
//	setsockopt(listenfd, SOL_SOCKET, SO_REUSEPORT, (void *) &one, sizeof(one));

	/* bind the name to the descriptor */
	Bind(listenfd, (struct sockaddr *)&servaddr, sizeof(servaddr));
	Listen(listenfd, 20);

	return (listenfd);
}
#endif

#if 1
/*
 * Wait for a client connection to arrive, and accept it.
 * We also obtain client`s user ID from the pathname
 * that it must bind before calling us.
 * Return new fd if all ok, <0 on error
 */
int CPublic::serv_accept(int listenfd)
{
	int connfd;
	struct sockaddr_in	cliaddr;
	socklen_t cliaddr_len;
	
	cliaddr_len = sizeof(cliaddr);
	if ((connfd = Accept(listenfd, (struct sockaddr *)&cliaddr, &cliaddr_len)) < 0)
		return -1; /* often errno = EINTER, if signal caught */

	return (connfd);
}
#endif

#if 1
/*
 *  Create a client endpoint of a connection.
 *   Return fd if all ok, <0 on error.
 */
int CPublic::cli_conn()
{
	int 	sockfd, rtn;
	struct sockaddr_in servaddr;

	/* create a UNIX domain stream socket */
	if ((sockfd = Socket(AF_INET, SOCK_STREAM, 0)) < 0)
		return -1;
	/* fill socket address structure with out address */
	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(SERV_PORT);
	inet_pton(AF_INET, "127.0.0.1", &servaddr.sin_addr);

	sleep(1);
	rtn = Connect(sockfd, (struct sockaddr *)&servaddr, sizeof(servaddr));
	if (rtn == -501)
		return  -501; // Agent stop

	return (sockfd);
}
#endif
int CPublic::Socket(int family, int type, int protocol)
{
	int n;

	if ((n = socket(family, type, protocol)) < 0)
		perr_exit("socket error");

	return n;
}

void CPublic::Listen(int fd, int backlog)
{
	if (listen(fd, backlog) < 0)
		perr_exit("listen error");
}

int CPublic::Connect(int fd, const struct sockaddr *sa, socklen_t salen)
{
	if (connect(fd, sa, salen) < 0)
		return -501;
}

void CPublic::Bind(int fd, const struct sockaddr *sa, socklen_t salen)
{
	if (bind(fd, sa, salen) < 0)
		perr_exit("bind error");
}

int CPublic::Accept(int fd, struct sockaddr *sa, socklen_t *salenptr)
{
	int n;

again:
	if ( (n = accept(fd, sa, salenptr)) < 0)
	{
		if ((errno == ECONNABORTED) || (errno == EINTR))
			goto again;
		else
			perr_exit("accept error");
	}
	return n;
}

/*
 * Read the socket descriptor.
 * Return the value is read, = -1 on error.
 */
int CPublic::Read(int fd, void *ptr, size_t nbytes)
{
	ssize_t n;

again:
	if ( (n = read(fd, ptr, nbytes)) == -1)
	{
		if (errno == EINTR)
			goto again;
		else
			return -1;
	}
	return n;
}

/*
 * Write the socket descriptor.
 * Return the value is writen, = -1 on error.
 */
int CPublic::Write(int fd, const void *ptr, size_t nbytes)
{
	ssize_t n;

again:
	if ( (n = write(fd, ptr, nbytes)) == -1)
	{
		if (errno == EINTR)
			goto again;
		else
			return -1;
	}
	return n;
}
void CPublic::Close(int fd)
{
	if (close(fd) == -1)
		perr_exit("close error");
}
void CPublic::perr_exit(const char *s)
{
	perror(s);
	exit(1);
}

void LogOneMsg(char * strMsg)
{
	FILE * pfile;
	char * strDate;
	char strtmp[32];
	char strPreface[32];
	time_t tm;
	int    ilong;
  
	time(&tm);
	strDate = ctime(&tm);

	ilong = strlen(strDate);
  
	strncpy(strtmp,strDate,ilong);
	strtmp[ilong-1] = 0;

	sprintf(strPreface,"%s : ",strtmp);
  
//  pfile = fopen("/home/tom/test.log","a");
//	pfile = fopen("/usr/lib/ssd/master/PAtest.txt","a");
    pfile = fopen("/home/leon/testSSD.log","a");
//  pfile = fopen("/home/tom/PAtest.log","a");
	if (!pfile)
	    return;

	fprintf(pfile,strPreface);
	fprintf(pfile,strMsg);
	fprintf(pfile,"\n");
  
	fclose(pfile);

	return;
}

