#include "ups.h"
#define PATH_USBLIB_LIBUSB		  AGENT_PATH"/UsbPort-Libusb.so.1.0.0"
#define PATH_USBLIB_HID		  AGENT_PATH"/UsbPort-HID.so.1.0.0"


#define SetStatus(status, bit)   	(status |= bit)
#define ClearStatus(status, bit) 	(status &= (0xFFFFFFFF - bit))
#define CheckBit(status, bit)    	((status & bit) == bit)


void CheckStatus(UINT uStatus, UINT uBit, UINT &uSet, UINT uSetBit)
{
	if (CheckBit(uStatus, uBit))
		SetStatus(uSet, uSetBit);
	else
		ClearStatus(uSet, uSetBit);
}

void DealWithAlrmSig(int iCount)
{
	signal(SIGALRM, DealWithAlrmSig);
}

int InitAlrmSig()
{
	struct itimerval value;
	signal(SIGALRM, DealWithAlrmSig);

	value.it_value.tv_sec =	3;
	value.it_value.tv_usec = 0;
	value.it_interval = value.it_value;
	setitimer(ITIMER_REAL, &value, NULL);

	return 0;
}

int EndAlrmSig()
{
	struct itimerval value;
	memset(&value, 0, sizeof(struct itimerval));
	setitimer(ITIMER_REAL, &value, NULL);

	return 0;
}

CUPS::CUPS()
{
	m_devType = USB;
	m_iComPortCount = 0;
	m_nOmronStatus = 0;
	m_nRestartTime = 0;
	memset(m_iPortNo, 0, sizeof(m_iPortNo));
	memset(&m_ShutdownParam, 0, sizeof(m_ShutdownParam));
}

CUPS::~CUPS()
{

}

int CUPS::Initialize_libusb()
{
	m_ifd = -1;
	m_dwUPSStatus = 0;
	const char *pszError = NULL;	
	m_nPortNum = -1;

	m_vLibHandle = LoadLibrary(PATH_USBLIB_LIBUSB);
	if (m_vLibHandle == NULL)
		return 0;

	LP_USBInitialize = (int(*)())dlsym(m_vLibHandle, "Initialize");
	if (pszError)
		return 0;

	LP_USBSearchUsb = (int(*)())dlsym(m_vLibHandle, "SearchUsb");
	if (pszError)
		return 0;

	LP_USBGetOneUsb = (int(*)())dlsym(m_vLibHandle, "GetOneUsb");
	if (pszError)
		return 0;

	LP_USBReadUsbPort = (int(*)(char *))dlsym(m_vLibHandle, "ReadUsbPort");
	if (pszError)
		return 0;

	LP_USBWriteUsbPort =(int(*)(const char *)) dlsym(m_vLibHandle, "WriteUsbPort");
	if (pszError)
		return 0;

	LP_USBClosePort = (void (*)())dlsym(m_vLibHandle, "ClosePort");
	if (pszError)
		return 0;

	LP_USBInitialize();

	return 1;
}

int CUPS::Initialize_hid()
{
	m_ifd = -1;
	m_dwUPSStatus = 0;
	const char *pszError = NULL;	
	m_nPortNum = -1;

	m_vLibHandle = LoadLibrary(PATH_USBLIB_HID);
	if (m_vLibHandle == NULL)
		return 0;

	LP_USBInitialize = (int(*)())dlsym(m_vLibHandle, "InitializeHid");
	pszError = dlerror();
	if (pszError)
		return 0;

	LP_USBSearchUsb = (int(*)())dlsym(m_vLibHandle, "SearchHIDUsb");
	pszError = dlerror();
	if (pszError)
		return 0;

	//	LP_OpenUPSUsbPort = (int(*)())dlsym(m_vLibHandle, "OpenUPSUsbPort");
	//	LP_GetVendorID = (int(*)(char *))dlsym(m_vLibHandle, "GetVendorID");
	LP_USBWriteUsbPort =(int(*)(const char *)) dlsym(m_vLibHandle, "SendCmdHIDUsb");
	pszError = dlerror();
	if (pszError)
		return 0;

	LP_USBReadUsbPort = (int(*)(char *))dlsym(m_vLibHandle, "ReadCmdHIDUsb");
	pszError = dlerror();
	if (pszError)
		return 0;

	LP_USBClosePort = (void (*)())dlsym(m_vLibHandle, "CloseHIDUsb");
	pszError = dlerror();
	if (pszError)
		return 0;

	LP_USBGetOneUsb = (int(*)())dlsym(m_vLibHandle, "GetOneUsb");
	pszError = dlerror();
	if (pszError)
		return 0;

	LP_USBInitialize();

	return 1;
}

//************************************//
//  Return value:
//      Return 1, succeed; 0, failed.
//  Description:
//      Open a COM or USB port.
//************************************//
int	CUPS::OpenConnection()
{
	m_nPortNum = LP_USBGetOneUsb();
	char szBuff[512] = {0};
	sprintf(szBuff, "m_nPortNum = %d", m_nPortNum);

	if (m_nPortNum > 0)
	{
		m_devType = USB;
		return 1;
	}
	else if (OpenOneCom() != -1)
	{
		m_nPortNum = m_iComNum;
		m_devType = COM;
		return 1;
	}

	return 0;
}

int	CUPS::OpenConnectionOnlyUSB()
{
	if ((m_nPortNum = LP_USBGetOneUsb()) > 0)
	{
		m_devType = USB;
		return 1;
	}	
	return 0;
}

//*******************************************************//
//  Return value:
//      If succeed, return the COM port file descriptor.
//  Description:
//      Open a COM port in auto searching way.
//*******************************************************//
int CUPS::OpenOneCom()
{
	int	 nIndex = 0;
	char szPortName[32];
	int	 arrPortNo[MAX_COM_COUNT];
	char szcmdBuf[MAX_UPS_DATA_LENGTH];
	int	 nCount = 0;
	struct termios comtios;
	
	m_devType = COM;
	nCount = GetComPortCount(arrPortNo);		

	if (nCount <= 0)
		return -1;

	while (nIndex < nCount)
	{
		sprintf(szPortName, "/dev/ttyS%d", m_iPortNo[nIndex++]);
		m_ifd = open(szPortName, O_RDWR | O_NOCTTY | O_NDELAY);	
		tcgetattr(m_ifd, &comtios);
		if (SetComAttribute(m_ifd) != -1)
		{	
			if (UPSCmdQ1(szcmdBuf) == 1)
				break;
			else
				tcsetattr(m_ifd, TCSANOW, &comtios);
		}
		if (m_ifd > 0)
			close(m_ifd);
	}

	m_iComNum = nIndex;  
	return m_ifd;
}

//***************************************************************************************//
//  Parameters:
//      nIndex
//          [in] The COM port index.
//  Return value:
//      Return the COM port file descriptor.
//  Description:
//      Establish connection with UPS through a COM port whose index is equal to nIndex.
//***************************************************************************************//
int CUPS::OpenSettedCom(int nIndex)
{
	char 	szPortName[32];
	char 	szcmdBuf[MAX_UPS_DATA_LENGTH];

	sprintf(szPortName, "/dev/ttyS%d", nIndex-1);

	if (-1 != m_ifd)
	{
		InitAlrmSig();
		close(m_ifd);
		EndAlrmSig();
		m_ifd = -1;
	}

	m_ifd = open(szPortName, O_RDWR | O_NOCTTY | O_NDELAY);	
	m_devType = COM;

	if (SetComAttribute(m_ifd) != -1)
	{
		if (UPSCmdQ1(szcmdBuf) == 1)
		{
			m_iComNum = nIndex;
			m_devType = COM;
			m_nPortNum = m_iComNum;

			return m_ifd;
		}
		else
			m_public.DelayInUsec(0, 500000);
	}

	if (m_ifd > 0)
		close(m_ifd);

	return 0;
}

//*********************************************************************//
//  Parameters:
//      fd
//          [in] The file descriptor of the COM port.
//      Ptios
//          [out] Get COM port attribute and store to termio structure.
//  Return value:
//      If return -1, failed, otherwise succeed.
//  Description:
//      This function is used to get the COM port attribute.
//*********************************************************************//
int CUPS::GetComAttribute(int fd, struct termios *ptios)
{
	if (fd != -1)
		return tcgetattr(fd, ptios);

	return -1;
}

//***************************************************//
//  Parameters:
//      fd
//          [in] The file descriptor of the COM port.
//  Return value:
//      Return -1, failed; 0, succeed.
//  Description:
//      Set the COM port attribute.
//***************************************************//
int CUPS::SetComAttribute(int fd)
{
	struct termios tios;

	if (fd == -1)
		return -1;

	if (tcgetattr(fd, &tios) == -1)
		return -1;

	tios.c_cflag &= ~(CBAUD | PARENB | CSTOPB);
	tios.c_cflag |= (B2400 | CS8 | CLOCAL);
	tios.c_iflag |= (IXON | IXOFF | ICRNL);
	tios.c_iflag &= ~(ISTRIP | IGNCR);
	tios.c_oflag &= ~(ONOCR | ONLCR);
	tios.c_lflag &= ~(ICANON | ECHO);
	tios.c_lflag |= FLUSHO;
	tios.c_cc[VMIN] = 1;
	tios.c_cc[VTIME] = 10;

	if (tcsetattr(fd, TCSANOW, &tios) == -1)
		return -1;

	return 0;	
}

//***************************************************//
//  Parameters:
//      fd
//          [in] The file descriptor of the COM port.
//      ptios
//          [in/out] The attribute set to COM port.
//  Return value:
//      If return -1, failed, otherwise succeed.
//  Description:
//      Reset the COM port attribute.
//***************************************************//
int CUPS::ResetComAttribute(int fd, struct termios *ptios)
{
	if (fd != -1)
		return tcsetattr(fd, TCSANOW, ptios);

	return -1;
}

void CUPS::ClosePort()
{
	if (m_devType == USB)
	{
		LP_USBClosePort();
		return;
	}
	else
	{
		if (m_ifd != -1)
		{
			InitAlrmSig();
			close(m_ifd);
			EndAlrmSig();
			m_ifd = -1;
		}
	}	
}

//*******************************************************************************//
//  Parameters:
//      iCount
//          [in] The length of pszUPSData in bytes.
//      pszUPSData
//          [out]	 A buffer for the incoming data.
//  Return value:
//      If successful, returns the number of bytes actually read, else return 0.
//  Description:
//  	This function is used to read incoming data from COM／USB port..
//*******************************************************************************//
int	CUPS::ReadPort(int iCount, char *pszUPSData)
{
	if (m_devType == USB)
	{
	    LogOneMsg(" ##leon - 	CUPS::ReadPort(int iCount, char *pszUPSData) - LP_USBReadUsbPort(pszUPSData)");
		return LP_USBReadUsbPort(pszUPSData);
	}

	int iByteRead = 0;
	int iTotalByte = 0;
	char *ptr = pszUPSData;
	fd_set readset;
	struct timeval tv;
	time_t tStartTime;
	CPublic Pub;

	if ((m_ifd == -1) || (pszUPSData == NULL))
		return 0;

	memset(pszUPSData, 0, MAX_UPS_DATA_LENGTH);
	tStartTime = time(NULL);

	//while((abs(time(NULL)-tStartTime) < 3))
	for (int retryCount = 0; retryCount < 5; ++retryCount)
	{
		tv.tv_sec = 0;
		tv.tv_usec = 500000;
		FD_ZERO(&readset);	
		FD_SET(m_ifd, &readset);

		if (select(m_ifd+1, &readset, NULL, NULL, &tv) > 0)
		{
			if (FD_ISSET(m_ifd, &readset))
			{
				iByteRead = read(m_ifd, ptr+iTotalByte, iCount);  
				if (iByteRead == -1)
					return 0;

				iTotalByte += iByteRead;
				if ((iTotalByte >= iCount) || strchr(ptr, '\n'))
					break;
			}
		}

		m_public.DelayInUsec(0, 500000);
	}

	return iTotalByte;
}

//*********************************************************************************//
//  Parameter:
//      pszData
//          [in] Data to be written to COM／USB port.
//  Return value:
//      If successful, return the number of bytes actually written, else return 0.
//  Description:
//      This function is used to write data to COM／USB port.
//*********************************************************************************//
int	CUPS::WritePort(const char *pszData)
{
	if (m_devType == USB)
	{
	    LogOneMsg(" ##leon - 	CUPS::WritePort(const char *pszData) - LP_USBWriteUsbPort(pszData)");
	    //1ms wait
	    m_public.DelayInUsec(0, 1000);
		return LP_USBWriteUsbPort(pszData);
	}

	char szCommand[MAX_COMMAND_LENGTH] = {0};
	int iRet = 0;

	if ((m_ifd == -1) || (pszData == NULL))
		return 0;

	strcpy(szCommand, pszData);

	szCommand[strlen(pszData)] = 0x0d;
	szCommand[strlen(pszData)+1] = 0x0;

	if ((iRet = write(m_ifd, szCommand, strlen(szCommand))) <= 0)
		return 0;

	return iRet;
}

//************************************************************************************************************************//
//  Parameters:
//      piOSPortNo
//          [out] All the COM ports index in the system.
//  Return value:
//      Return the count of COM port of the system.
//  Description:
//      Get the total number of available COM port in the system, and keep the COM index into an array pointer piOSPortNo.
//************************************************************************************************************************//
int CUPS::GetComPortCount(int *piOSPortNo)
{
	int ifd = 0;
	int i = 0;
	int iCount = 0;
	int iPortNo = 0;
	int *piPortNo = NULL;
	struct termios comtios;
	char szPortName[32] = {0};

	piPortNo = m_iPortNo;

	while (1)
	{
		memset(szPortName, 0, sizeof(szPortName));
		sprintf(szPortName, "/dev/ttyS%d", iPortNo);

		if ((ifd = open(szPortName, O_RDWR | O_NOCTTY | O_NDELAY)) < 0)
			break;

		if (GetComAttribute(ifd, &comtios) != -1)
		{
			if (SetComAttribute(ifd) != -1)
				piPortNo[iCount++] = iPortNo;

			ResetComAttribute(ifd, &comtios);
		}
		close(ifd);

		iPortNo++;
		if (iPortNo >= 256)
			break;
	}

	m_iComPortCount = iCount;

	if (piOSPortNo != NULL)
	{
		for (i = 0; i < iCount; i++)
		{
			piOSPortNo[i] = m_iPortNo[i];
		}
	}

	return iCount;
}

//************************************//
//  Return value:
//      Return 1, succeed; 0, failed.
//  Description:
//      Read data “OK” from UPS.
//************************************//
int CUPS::UPSReadOk()
{
	char szData[MAX_UPS_DATA_LENGTH];
	memset(szData, 0, sizeof(szData));

	if (ReadPort(3, szData) > 0)
	{
		if (strlen(szData) == 3)
		{
			if ((szData[0] == 'O') && (szData[1] == 'K'))
				return 1;
		}
	}

	return 0;
}

//*************************************************************************************//
//  Parameters:
//      szData
//          [out] Storage location of data from UPS.
//  Return value:
//      Return 1, succeed; 0, failed.
//  Description:
//      Send "Q1" command to UPS, and store the data received from UPS into (szData).
//*************************************************************************************//
int CUPS::UPSCmdQ1(char *szData)
{
	UINT i;
	for (i = 0; i < MAX_PORT_FAIL_LOOP; i++)
	{
		if (WritePort("Q1") < 0)
			continue;

		if (ReadPort(47, szData) > 0)
		{
			if (strlen(szData) == 47)
				return 1;
		}
	}

	ClosePort();
	return 0;
}

//*******************************************************************//
//  Parameters:
//      uDown
//          [in] Time parameter of shutdown UPS command.
//  Return value:
//      return 1, succeed; 0, failed
//  Description:
//      send “Sn” Command to UPS, if read “OK” from UPS, succeed.
//*******************************************************************//
int CUPS::UPSCmdSN(UINT uDown)
{
	int  i = 0;
	char szCommand[31];
	char szTemp[512];
	char szCommand2[31];

	if (m_nOmronStatus == UPS_STATUS_ONLY_SD_OS)
		return TRUE;

	memset(szCommand, 0, sizeof(szCommand));
	if (uDown > 1200)
		uDown = 1200;
	if (uDown < 12)
		uDown = 12;
	if (uDown >= 60)
		sprintf(szCommand, "S%02u", uDown/60);
	else
		sprintf(szCommand, "S.%01u", uDown/6);

	if (m_nOmronStatus == UPS_STATUS_SD_OS_RESTART_LATER)
	{
		memset(szCommand2, 0, sizeof(szCommand2));
		m_nRestartTime = m_nRestartTime - (uDown/60);
		if (m_nRestartTime < 1)
		    m_nRestartTime = 1;
		sprintf(szCommand2, "%sR%05u", szCommand, m_nRestartTime);
	}
	else if ( m_nOmronStatus == UPS_STATUS_SD_OS_UPS)
	{
		memset(szCommand2, 0, sizeof(szCommand2));

		if (uDown >= 60)
			sprintf(szCommand2, "Sf%02u", uDown/60);
		else
			sprintf(szCommand2, "Sf.%01u", uDown/6);
	}
	else
	{
		memset(szCommand2, 0, sizeof(szCommand2));
		strcpy(szCommand2, szCommand);
	}

	for (i = 0; i < MAX_PORT_FAIL_LOOP; i++)
	{
		if (WritePort(szCommand2) > 0)
		{
			if (UPSReadOk() == 1)
				return 1;
		}
	}
	
	ClosePort(); 
	return FALSE;
}

#if 0
int CUPS::UPSCmdSN(UINT uDown)
{
	int  i = 0;
	char szCommand[15];
	memset(szCommand, 0, sizeof(szCommand));

	if (uDown > 600)
		uDown = 600;
	if (uDown < 12)
		uDown = 12;	
	if (uDown >= 60)
		sprintf(szCommand, "S%02u", uDown/60);
	else
		sprintf(szCommand, "S.%01u", uDown/6);

	for (i = 0; i < MAX_PORT_FAIL_LOOP; i++)
	{
		if (WritePort(szCommand) > 0)
		{
			if (UPSReadOk() == 1)
				return 1;
		}
	}
	ClosePort(); 
	return 0;
}
#endif

//*****************************************************//
//  Return value:
//      If succeed in establishing connection, return 1.
//  Description:
//      Establish connection with UPS.
//*****************************************************//
int CUPS::ConnectUPS()
{
	char szData[MAX_UPS_DATA_LENGTH];
	memset(szData, 0, MAX_UPS_DATA_LENGTH);
	
	if (OpenConnection() != -1)
	{
		if (m_devType == COM)
		{
			if (SetComAttribute(m_ifd) == -1)
			{
				ClosePort();
				return 0;
			}
		}

		if (UPSCmdQ1(szData) == 1)
		{
//			ClosePort();//tom added on 2010-1-22 17:40
			return 1;
		}
		
		ClosePort();
	}

	return 0;
}

int CUPS::ConnectUPSOnlyUSB()
{
	char szData[MAX_UPS_DATA_LENGTH];
	memset(szData, 0, MAX_UPS_DATA_LENGTH);
	m_devType = USB;	
	if (OpenConnectionOnlyUSB())
	{
		if (m_devType == COM)
		{
			if (SetComAttribute(m_ifd) == -1)
			{
				ClosePort();
				return 0;
			}
		}

		if (UPSCmdQ1(szData) == 1)
			return 1;

		ClosePort();
	}

	return 0;
}

//**************************************************************************//
//  Return value:
//      If failed, return 0; otherwise return 1.
//  Description:
//      Send "Q1" command to UPS, and get the current status of UPS to judge
//      if AC fail or Battery Low event happened or not.
//**************************************************************************//
int CUPS::PollingUPS()
{
	UINT uStatus;
	char szData[MAX_UPS_DATA_LENGTH] = { 0 };
	char s1[10] = { 0 }, s2[10] = { 0 }, s3[10] = { 0 }, s4[10] = { 0 };
	char s5[10] = { 0 }, s6[10] = { 0 }, s7[10] = { 0 };

	memset(szData, 0, sizeof(szData));

	if (UPSCmdQ1(szData) == 0)
		return 0;

	sscanf(szData, "(%s %s %s %s %s %s %s %8x\n", s1, s2, s3, s4, s5, s6, s7, &uStatus);

	CheckStatus(uStatus, UPS_STATUS_BATTLOW_Q1,	m_dwUPSStatus, UPS_STATUS_BATTLOW);
	CheckStatus(uStatus, UPS_STATUS_ACFAIL_Q1,	m_dwUPSStatus, UPS_STATUS_ACFAIL);

	return 1;
}

DWORD CUPS::GetUPSStatus()
{
	return m_dwUPSStatus;
}

//*****************************************************//
//  Parameters:
//      pszPath
//          [in] The file path of the dll to be opened.
//  Return value:
//      Return the handle of the dll.
//  Description:
//      Load the dynamic library file named by pszPath.
//*****************************************************//
void * CUPS::LoadLibrary(char *pszPath)
{
	void *dlhandle = NULL;
	const char *pszError = NULL;

	dlhandle = dlopen(pszPath, RTLD_NOW);
	pszError = dlerror();

	if (pszError)
		return NULL;

	return dlhandle;
}
