#ifndef __UPS_H
#define __UPS_H

#ifdef __cplusplus
extern "C" {
#endif

#include "stdpro_linux.h"
#include "Public.h"

#define UPS_MAX					256
#define MAX_UPS_DATA_LENGTH		128
#define MAX_COM_COUNT			128

#define MAX_PORT_FAIL_LOOP		2
#define MAX_COMMAND_LENGTH		12

#define UPS_STATUS_BATTLOW_Q1	0x1000000
#define UPS_STATUS_ACFAIL_Q1		0x10000000

#define UPS_STATUS_BATTLOW		0x1
#define UPS_STATUS_ACFAIL		0x2

int	InitAlrmSig();
void	DealWithAlrmSig(int iCount);
int	EndAlrmSig();

class CUPS
{
public:
	CUPS();
	~CUPS();
	int		Initialize_libusb();
	int 		Initialize_hid();

	int		ConnectUPS();
	int		ConnectUPSOnlyUSB();
	int		PollingUPS();
	void		ClosePort();
	int		UPSCmdQ1(char *szData);
	int		UPSCmdSN(UINT uDown);
	int		GetComPortCount(int *piOSPortNo = NULL);
	DWORD	GetUPSStatus();
	int		OpenSettedCom(int nIndex);

private:	
	int		GetComAttribute(int fd, struct termios *ptios);
	int		SetComAttribute(int fd);
	int		ResetComAttribute(int fd, struct termios *ptios);
	
	int		OpenConnection();
	int		OpenConnectionOnlyUSB();
	int		OpenOneCom();
	int		ReadPort(int iCount, char *pszUPSData);
	int		WritePort(const char *pszData);
	int		UPSReadOk();
	
	void		*LoadLibrary(char *pszData);

public:
	SHUTDOWNPARAM	m_ShutdownParam;

	int		m_ifd;
	int		m_iComPortCount;
	int		m_iPortNo[UPS_MAX];
	int		m_iComNum;
	int		m_nPortNum;
	DWORD	m_dwUPSStatus;
	
	PortType	m_devType;
	CPublic	m_public;
	int			m_nOmronStatus;
	int			m_nRestartTime;
private:
	void *	m_vLibHandle;
	int		(*LP_USBInitialize)();
	int   	(*LP_USBSearchUsb)();
	int		(*LP_USBGetOneUsb)();
	int   	(*LP_USBReadUsbPort)(char *pszData);
	int   	(*LP_USBWriteUsbPort)(const char *pszUPSData);
	void	(*LP_USBClosePort)();
};


#ifdef __cplusplus
}
#endif

#endif
