#ifndef _UPSKERNEL_H
#define _UPSKERNEL_H

#ifdef __cplusplus
extern "C" {
#endif

#include 	"../Public/Public.h"
#include 	"../Public/ConfigFile.h"
#include 	"../Public/Event.h"

const char	SHUTDOWNCMD[] = "/sbin/halt -p"; //fixed bug:Machine cannot power off by Simple Shutdown. (CentOS 7, Red Hat Enterprise Linux 7)
const char	REBOOTCMD[] = "/sbin/reboot";

const char 	Hibernation[] = "echo disk > /sys/power/state";
const char 	Hibernation_centos7[] = "systemctl hibernate";

#define UPS_STATUS_BATTLOW 	0x1
#define UPS_STATUS_ACFAIL 	0x2
#define ALL_KVM_STOP	0x0

class CUpsKernel
{
public:
	CConfigFile			m_fileConfig;
	CPublic				m_public;
	CEvent				m_event;
	
	SHUTDOWNPARAM		m_SDParam;	
	PSHM_DATA			m_pSHMData;
	PIPE_DATA			m_SendPipe;
	PIPE_DATA			m_RecvPipe;
	int					m_nFileLock;
	DWORD				m_dwUPSStatus;
	int 				RebootSys;

	
private:
	int			m_writefifofd;
	int			m_writefifoTmp;
	int			m_readfifofd;
	
	int			m_nService;
	int			m_nACFail;
	time_t		m_tACFailStart;
	int			m_nFirstCommEst;
	int			m_nFirstBattLow;
	int			m_nFirstACFail;
	int			m_iShutdownStep;
	time_t		m_tStartShutdownTime;
	int 		kvm_num;
	int 		m_kvmsetp;
#if 0
	int 		m_nOmronStatus;
	int 		m_nRestartTime;
#endif
		
public:
	CUpsKernel(void);
	~CUpsKernel(void);
	
public:
	int Initialize();
	void Run();
	void TermService();
	
private:
	int		InitializeService();
	int		CheckEvent();
	int		CheckBattLow();
	int		CheckACFail();
	int		DoDelay();
	int		DoShutdownLogic();
	int 	GetKVMState();
	void 	CloseKVM();
	void 	StartKVM();
	int  		Systembit();
	void 	DelKVMSave();

	void	ShutdownSystem();
	
	void	RebootSystem();
	void 	HibernationSystem();
	int		StartProcess(char *pszProcessName);
	void 	PraseStringStr(char *str);
	void 	PraseStringNum(char *str, int *arr);

	static CUpsKernel 	*m_pThis;
	static void 		sig_usr(int sig);
};

#ifdef __cplusplus
}
#endif

#endif
