/*
 *	colin modify the omrctl.cpp to support Linux system.
 *	Omronctl use local socket to communicate with SSDDemon.
 *	Linux versions omrctl consistent with the Windows version.
 *
 */
#include "omronctl.h"
CPublic	m_socket;
CEvent 	m_event;
CConfigFile m_config;	
SHUTDOWNPARAM ShutdownParam;


BOOL CheckTimeFormat(const char *pszTime)
{
	char szTemp[512];

	char szYear[4 + 1] = {0};
	char szMon[2 + 1] = {0};
	char szDay[2 + 1] = {0};
	char szHour[2 + 1] = {0};
	char szMin[2 + 1] = {0};

	int nYear = 0;
	int nMon = 0;
	int nDay = 0;
	int nHour = 0;
	int nMin = 0;

	int i = 0;

	memset(szTemp, 0x00, sizeof(szTemp));
	strcpy(szTemp, pszTime);

	if (strlen(szTemp) != 16)
		return FALSE;

	if ( szTemp[4] != '/' || szTemp[7] != '/' || szTemp[10] != '.' || szTemp[13] != ':' )
		return FALSE;

	for (i = 0; i < 4; i ++)
	{
		szYear[i] = szTemp[i];
	}
	for (i = 0; i < 2; i ++)
	{
		szMon[i] = szTemp[i + 5];
		szDay[i] = szTemp[i + 8];
		szHour[i] = szTemp[i + 11];
		szMin[i] = szTemp[i + 14];
	}
	nYear = GetInpuId(szYear);
	nMon = GetInpuId(szMon);
	nDay = GetInpuId(szDay);
	nHour = GetInpuId(szHour);
	nMin = GetInpuId(szMin);

	if (nYear < 1970 || nYear > 2100 ||
			nMon < 1 || nMon > 12 ||
			nDay < 1 || nDay > 31 ||
			nHour < 0 || nHour >= 24 || 
			nMin < 0 || nMin > 59)
	{
		return FALSE;
	}
	switch(nMon)
	{
		case 1:
		case 3:
		case 5:
		case 7:
		case 8:
		case 10:
		case 12:
			break;
		default:
			if (nDay > 30)
			{
				return FALSE;
			}
			break;
	}

	return TRUE;
}


int GetLaterTimeMin(const char *pszTime)
{
	tm structtm;

	char szTemp[512];

	char szYear[4 + 1] = {0};
	char szMon[2 + 1] = {0};
	char szDay[2 + 1] = {0};
	char szHour[2 + 1] = {0};
	char szMin[2 + 1] = {0};

	int i = 0;
	int nTimeLater = 0;

	memset(szTemp, 0x00, sizeof(szTemp));
	strcpy(szTemp, pszTime);

	if (strlen(szTemp) != 16)
		return 0;

	if ( szTemp[4] != '/' || szTemp[7] != '/' || szTemp[10] != '.' || szTemp[13] != ':' )
		return FALSE;

	for (i = 0; i < 4; i ++)
	{
		szYear[i] = szTemp[i];
	}
	for (i = 0; i < 2; i ++)
	{
		szMon[i] = szTemp[i + 5];
		szDay[i] = szTemp[i + 8];
		szHour[i] = szTemp[i + 11];
		szMin[i] = szTemp[i + 14];
	}
	memset(&structtm, 0, sizeof(structtm));
	structtm.tm_year = GetInpuId(szYear);
	structtm.tm_year = structtm.tm_year -1900;
	structtm.tm_mon =  GetInpuId(szMon);
	structtm.tm_mon = structtm.tm_mon -1;
	structtm.tm_mday = GetInpuId(szDay);
	structtm.tm_hour = GetInpuId(szHour);
	structtm.tm_min = GetInpuId(szMin);
	structtm.tm_isdst = 1;	

	time_t	ctNow = time(NULL);
	time_t	ctUserTime = mktime(&structtm);

	nTimeLater = ctUserTime - ctNow;

	//nTimeLater = ctTimeInterval.GetTotalSeconds();

	if (nTimeLater < 0)
		return -1;

	return nTimeLater/60 ;

}

time_t StructIntoTime(int nYear, int nMoon, int nDay, int nHour, int nMin, int nSec)
{
	struct tm tTempTime;

	memset(&tTempTime, 0, sizeof(struct tm));
	tTempTime.tm_sec = nSec;
	tTempTime.tm_min = nMin;
	tTempTime.tm_hour = nHour;
	tTempTime.tm_mday = nDay;
	tTempTime.tm_mon = nMoon-1; /* range 0 to 11 */
	tTempTime.tm_year = nYear-1900; /* since 1900 */
	tTempTime.tm_isdst = 1;	

	return mktime(&tTempTime);
}

int GetInpuId(const char *szInput)
{
	int nInput = 0;
	char szBuff[MAX_PATH];
	char szTemp[512];
	BOOL bFirstNotZero = FALSE;
	int nFirstNotZero = 0;
	int i = 0;
	nInput = atoi(szInput);

	memset(szTemp, 0x00, sizeof(szTemp));
	for (i = 0; i < strlen(szInput); i ++)
	{
		if (!bFirstNotZero && szInput[i] != '0')
		{
			bFirstNotZero = TRUE;
			nFirstNotZero = i;
		}
		if (bFirstNotZero)
			szTemp[i - nFirstNotZero] = szInput[i];
	}
	if (!bFirstNotZero)
		szTemp[0] = '0';

	memset(szBuff, 0, MAX_PATH);
	sprintf(szBuff, "%d", nInput);
	if (strcmp(szTemp, szBuff) != 0 || nInput < 0)
		nInput = -1;
	return nInput;
}


/* Send S<n>R<m> comand for JP1. <m> is set by ParamO [seconds] */
int DoCommandSock(int nOperationSign, int ParamO)
{
	int	ret = 0;
	int	sockfd;
	int socketrtn;
	SOCKET_DATA_OMRON socketdata;
	char sendbuf[sizeof(socketdata)] = {0};

	sockfd = m_socket.cli_conn();
	if (sockfd > 0)
	{
		memset(&socketdata, 0x00, sizeof(socketdata));
		socketdata.uCmd   = nOperationSign;
		socketdata.uPara1 = 0;
		socketdata.uPara2 = ParamO;

		memcpy(sendbuf,&socketdata,sizeof(socketdata)); // convert struct to string
		if ((socketrtn = m_socket.Write(sockfd, sendbuf, sizeof(socketdata))) < 0)  // Send struct to ssdDemon
			return ERR_OTHER;

		memset(sendbuf, 0, sizeof(sendbuf));
		m_socket.Read(sockfd , sendbuf, sizeof(socketdata));
		memcpy(&socketdata, sendbuf, sizeof(socketdata)); //convert string to structure
		if (socketdata.uPara3 == ERR_COMLST)
			return ERR_COMLST;
		else if (socketdata.uPara4 == ERR_COMMEXEC)
			return ERR_COMMEXEC;
	}
	else if (sockfd == -501)
		return ERR_AGENTSTOP;

	return ret;
}


void ShowHelp()
{
	printf("    Simple Shutdown 2.40 (Release Date:2017-03-29)\n");
	printf("    2011 OMRON Corporation and Ingrasys Technology Inc.\n");
	printf("    All rights Reserved.\n\n\n");

	printf("    HELP:\n");
	printf("    1, omronctl (No return value)\n");   //modify 2011824
	printf("       <Function>\n");
	printf("          Shutdown OS and UPS.\n\n");

	printf("    2, omronctl -O YYYY/MM/DD.hh:mm\n");
	printf("       <Function>\n");
	printf("          Shutdown OS and UPS and then restart UPS at the set time.\n\n");

	printf("    3, omronctl /f /r\n");
	printf("       <Function>\n");
	printf("          Run this command and OS shutdown and restart system.\n\n");
	return ;
}


void PrintWarningInfo()
{
	printf("\n    The Command is executing.\n");
}

void PrintSuccInfo()
{
	printf("\n    The Command Successful.\n");
}


int main(int argc, char* argv[])
{
	int ret = 0;
	int nTimeMinLater = 0;
	char tmp_time[128] = {0};
	//SOCKET_DATA_OMRON Socketdata;

	m_config.GetShutdownParameter(&ShutdownParam);

	/* Ignore Ctrl-C */
	//signal(SIGINT, SIG_IGN); 

	if (argc == 1)
	{
		int tmp = DoCommandSock(UPS_STATUS_SD_OS_UPS, 0);
		if ( tmp != ERR_OTHER && tmp != ERR_COMMEXEC && tmp != ERR_COMLST && tmp != ERR_AGENTSTOP)
		{
			PrintSuccInfo();
			m_event.WriteEventLog(pStrEvent[17]);
			if (ShutdownParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[17]);
			return 0;
		}
		else if(tmp == ERR_OTHER)
		{
			m_event.WriteEventLog(pStrEvent[16]);
			if (ShutdownParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[16]);
		}
		else if (tmp == ERR_COMLST)
		{
			m_event.WriteEventLog(pStrEvent[12]);
			if (ShutdownParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[12]);
			printf("\n    SimpleShutdown communication UPS lost.\n");
			printf("    Please check the UPS is connected.\n\n");

		}
		else if (tmp == ERR_COMMEXEC)
		{
			m_event.WriteEventLog(pStrEvent[13]);
			if (ShutdownParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[13]);
			PrintWarningInfo();
		}
		else if (tmp == ERR_AGENTSTOP)
		{
			printf("\n	  Can not communicate with Simple Shutdown Service.\n");
			printf("    Please check the service is running.\n\n");
		}

		return tmp;
	}
	else if ((argc == 3) &&	strcmp( argv[1], "-O") == 0 && CheckTimeFormat( argv[2] ))
	{
		nTimeMinLater = GetLaterTimeMin( argv[2] );

		if (nTimeMinLater < 0)
		{
			printf("     Time setting is Wrong.\n");
			m_event.WriteEventLog(pStrEvent[14]);
			if(ShutdownParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[14]);

			ShowHelp();

			return ERR_TIMEWRONG;
		}		
		{
			if (nTimeMinLater <= 99999)
			{
				int realshutdowntime = (ShutdownParam.uShutdownNeedTime + ShutdownParam.uExCmdNeedTime) / 60;
				if ((nTimeMinLater - realshutdowntime) > 0)
				{
					int tmp = DoCommandSock(UPS_STATUS_SD_OS_RESTART_LATER, nTimeMinLater);
					if (tmp != ERR_OTHER && tmp != ERR_COMMEXEC && tmp != ERR_COMLST && tmp != ERR_AGENTSTOP)
					{
						PrintSuccInfo();
						sprintf(tmp_time, pStrEvent[18], nTimeMinLater);
						m_event.WriteEventLog(tmp_time);
						if(ShutdownParam.iSendMsg)
							m_event.BroadcastMessage(tmp_time);
						return 0;
					}
					else if(tmp == ERR_OTHER)
					{
						m_event.WriteEventLog(pStrEvent[16]);
						if(ShutdownParam.iSendMsg)
							m_event.BroadcastMessage(pStrEvent[16]);
					}
					else if (tmp == ERR_COMLST) {
						m_event.WriteEventLog(pStrEvent[12]);
						if(ShutdownParam.iSendMsg)
							m_event.BroadcastMessage(pStrEvent[12]);
						printf("\n    SimpleShutdown communication UPS lost.\n");
						printf("    Please check the UPS is connected.\n\n");
					}
					else if (tmp == ERR_COMMEXEC)
					{
						m_event.WriteEventLog(pStrEvent[13]);
						if(ShutdownParam.iSendMsg)
							m_event.BroadcastMessage(pStrEvent[13]);
						PrintWarningInfo();
					}
					else if (tmp == ERR_AGENTSTOP)
					{
						printf("\n	  Can not communicate with Simple Shutdown Service.\n");
						printf("    Please check the service is running.\n\n");
					}
					return tmp;
				}

				else
				{
					int tmp = DoCommandSock(UPS_STATUS_SD_OS_RESTART_LATER, realshutdowntime+1);
					if (tmp != ERR_OTHER && tmp != ERR_COMMEXEC && tmp != ERR_COMLST && tmp != ERR_AGENTSTOP)
					{
						PrintSuccInfo();
						sprintf(tmp_time, pStrEvent[18], realshutdowntime+1);
						m_event.WriteEventLog(tmp_time);
						if(ShutdownParam.iSendMsg)
							m_event.BroadcastMessage(tmp_time);
						return 0;
					}
					else if(tmp == ERR_OTHER)
					{
						m_event.WriteEventLog(pStrEvent[16]);
						if(ShutdownParam.iSendMsg)
							m_event.BroadcastMessage(pStrEvent[16]);
					}
					else if (tmp == ERR_COMLST)
					{
						m_event.WriteEventLog(pStrEvent[12]);
						if(ShutdownParam.iSendMsg)
							m_event.BroadcastMessage(pStrEvent[12]);
						printf("\n    SimpleShutdown communication UPS lost.\n");
						printf("    Please check the UPS is connected.\n\n");
					}
					else if (tmp == ERR_COMMEXEC)
					{
						m_event.WriteEventLog(pStrEvent[13]);
						if(ShutdownParam.iSendMsg)
							m_event.BroadcastMessage(pStrEvent[13]);
						PrintWarningInfo();
					}
					else if (tmp == ERR_AGENTSTOP)
					{
						printf("\n	  Can not communicate with Simple Shutdown Service.\n");
						printf("    Please check the service is running.\n\n");
					}
					return tmp;
/*
					m_event.WriteEventLog(pStrEvent[15]);
					if(ShutdownParam.iSendMsg)
						m_event.BroadcastMessage(pStrEvent[15]);
					printf("\n");
					printf("    Can't do the schedule shutdown action.\n");

					return WARR_REATART_TIME;

*/
				}
			}
			else
			{
				m_event.WriteEventLog(pStrEvent[14]);
				if (ShutdownParam.iSendMsg)
					m_event.BroadcastMessage(pStrEvent[14]);
				printf("\n");
				printf("    Time setting is Wrong.\n\n");

				return  ERR_TIMEWRONG;
			}

		}
	}
	else if ((argc == 3) && strcmp(argv[1], "/f") == 0 &&strcmp(argv[2], "/r") == 0)
	{
		int tmp = DoCommandSock(UPS_STATUS_ONLY_SD_OS, 0);
		if ( tmp != ERR_OTHER && tmp != ERR_COMMEXEC && tmp != ERR_COMLST && tmp != ERR_AGENTSTOP)
		{
			PrintSuccInfo();
			m_event.WriteEventLog(pStrEvent[19]);
			if(ShutdownParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[19]);
			return 0;
		}
		else if(tmp == ERR_OTHER)
		{
			m_event.WriteEventLog(pStrEvent[16]);
			if(ShutdownParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[16]);
		}
		else if (tmp == ERR_COMMEXEC)
		{
			m_event.WriteEventLog(pStrEvent[13]);
			if (ShutdownParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[13]);
			PrintWarningInfo();
			ret = ERR_COMMEXEC;
		}
		else if (tmp == ERR_COMLST)
		{
			m_event.WriteEventLog(pStrEvent[12]);
			if (ShutdownParam.iSendMsg)
				m_event.BroadcastMessage(pStrEvent[12]);
			printf("\n    SimpleShutdown communication UPS lost.\n");
			printf("    Please check the UPS is connected.\n\n");
		}
		else if (tmp == ERR_AGENTSTOP)
		{
			printf("\n	  Can not communicate with Simple Shutdown Service.\n");
			printf("    Please check the service is running.\n\n");
		}

		return tmp;
	}
	else if (!((argc == 2) && strcmp(argv[1], "/?") == 0 ))
	{
		m_event.WriteEventLog(pStrEvent[14]);
		if (ShutdownParam.iSendMsg)
			m_event.BroadcastMessage(pStrEvent[14]);
		printf("\n");
		printf("    Parameter Format is Wrong.\n\n");

		ret = ERR_TIMEWRONG;
	}

	ShowHelp();
	return ret;
}

